#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import print_function

import os
import sys

import six
import uuid

from oslo_serialization import jsonutils
from oslo_utils import encodeutils

import prettytable

from karborclient.common.apiclient import exceptions


# Decorator for cli-args
def arg(*args, **kwargs):
    def _decorator(func):
        # Because of the sematics of decorator composition if we just append
        # to the options list positional options will appear to be backwards.
        func.__dict__.setdefault('arguments', []).insert(0, (args, kwargs))
        return func
    return _decorator


def env(*vars, **kwargs):
    """Search for the first defined of possibly many env vars

    Returns the first environment variable defined in vars, or
    returns the default defined in kwargs.
    """
    for v in vars:
        value = os.environ.get(v)
        if value:
            return value
    return kwargs.get('default', '')


def _print(pt, order):
    if sys.version_info >= (3, 0):
        print(pt.get_string(sortby=order))
    else:
        print(encodeutils.safe_encode(pt.get_string(sortby=order)))


def print_list(objs, fields, exclude_unavailable=False, formatters=None,
               sortby_index=0):
    '''Prints a list of objects.

    @param objs: Objects to print
    @param fields: Fields on each object to be printed
    @param exclude_unavailable: Boolean to decide if unavailable fields are
                                removed
    @param formatters: Custom field formatters
    @param sortby_index: Results sorted against the key in the fields list at
                         this index; if None then the object order is not
                         altered
    '''
    formatters = formatters or {}
    mixed_case_fields = ['serverId']
    removed_fields = []
    rows = []

    for o in objs:
        row = []
        for field in fields:
            if field in removed_fields:
                continue
            if field in formatters:
                row.append(formatters[field](o))
            else:
                if field in mixed_case_fields:
                    field_name = field.replace(' ', '_')
                else:
                    field_name = field.lower().replace(' ', '_')
                if type(o) == dict and field in o:
                    data = o[field]
                else:
                    if not hasattr(o, field_name) and exclude_unavailable:
                        removed_fields.append(field)
                        continue
                    else:
                        data = getattr(o, field_name, '')
                if data is None:
                    data = '-'
                if isinstance(data, six.string_types) and "\r" in data:
                    data = data.replace("\r", " ")
                row.append(data)
        rows.append(row)

    for f in removed_fields:
        fields.remove(f)

    pt = prettytable.PrettyTable((f for f in fields), caching=False)
    pt.align = 'l'
    for row in rows:
        pt.add_row(row)

    if sortby_index is None:
        order_by = None
    else:
        order_by = fields[sortby_index]
    _print(pt, order_by)


def print_dict(d, property="Property", dict_format_list=None,
               json_format_list=None):
    pt = prettytable.PrettyTable([property, 'Value'], caching=False)
    pt.align = 'l'
    for r in d.items():
        r = list(r)
        if isinstance(r[1], six.string_types) and "\r" in r[1]:
            r[1] = r[1].replace("\r", " ")
        if dict_format_list is not None and r[0] in dict_format_list:
            r[1] = dict_prettyprint(r[1])
        if json_format_list is not None and r[0] in json_format_list:
            r[1] = json_prettyprint(r[1])
        pt.add_row(r)
    _print(pt, property)


def dict_prettyprint(val):
    """dict pretty print formatter.

    :param val: dict.
    :return: formatted json string.
    """
    return jsonutils.dumps(val, indent=2, sort_keys=True)


def json_prettyprint(val):
    """json pretty print formatter.

    :param val: json string.
    :return: formatted json string.
    """
    return val and jsonutils.dumps(jsonutils.loads(val),
                                   indent=2, sort_keys=True)


def find_resource(manager, name_or_id, *args, **kwargs):
    """Helper for the _find_* methods."""
    # first try to get entity as integer id
    try:
        if isinstance(name_or_id, int) or name_or_id.isdigit():
            return manager.get(int(name_or_id), *args, **kwargs)
    except exceptions.NotFound:
        pass

    # now try to get entity as uuid
    try:
        uuid.UUID(str(name_or_id))
        return manager.get(name_or_id, *args, **kwargs)
    except (ValueError, exceptions.NotFound):
        pass

    # finally try to find entity by name
    try:
        return manager.find(name=name_or_id)
    except exceptions.NotFound:
        msg = "No %s with a name or ID of '%s' exists." % \
              (manager.resource_class.__name__.lower(), name_or_id)
        raise exceptions.CommandError(msg)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from oslo_serialization import jsonutils

from karborclient.osc.v1 import checkpoints as osc_checkpoints
from karborclient.tests.unit.osc.v1 import fakes
from karborclient.v1 import checkpoints


CHECKPOINT_INFO = {
    "id": "dcb20606-ad71-40a3-80e4-ef0fafdad0c3",
    "project_id": "e486a2f49695423ca9c47e589b948108",
    "status": "available",
    "protection_plan": {
        "id": "3523a271-68aa-42f5-b9ba-56e5200a2ebb",
        "name": "My application",
        "provider_id": "cf56bd3e-97a7-4078-b6d5-f36246333fd9",
        "resources": [{
            "id": "99777fdd-8a5b-45ab-ba2c-52420008103f",
            "type": "OS::Glance::Image",
            "name": "cirros-0.3.4-x86_64-uec"}]
    },
    "resource_graph": jsonutils.dumps(
        "[{'0x0': ['OS::Glance::Image', "
        "'99777fdd-8a5b-45ab-ba2c-52420008103f', "
        "'cirros-0.3.4-x86_64-uec']}, [[['0x0']]]]"
    ),
}

CHECKPOINT_INFO_2 = {
    "id": "a6fd95fe-0892-43b2-ad3c-e56f3a1b86b8",
    "project_id": "79b35e99a6a541b3bcede40f590d6878",
    "status": "available",
    "protection_plan": {
        "id": "3b47fd5d-21f9-4e63-8409-0acb1bffc038",
        "name": "My application",
        "provider_id": "cf56bd3e-97a7-4078-b6d5-f36246333fd9",
        "resources": [{
            "id": "99777fdd-8a5b-45ab-ba2c-52420008103f",
            "type": "OS::Glance::Image",
            "name": "cirros-0.3.4-x86_64-uec"}]
    },
    "resource_graph": jsonutils.dumps(
        "[{'0x0': ['OS::Glance::Image', "
        "'99777fdd-8a5b-45ab-ba2c-52420008103f', "
        "'cirros-0.3.4-x86_64-uec']}, [[['0x0']]]]"
    ),
}


class TestCheckpoints(fakes.TestDataProtection):
    def setUp(self):
        super(TestCheckpoints, self).setUp()
        cm = self.app.client_manager
        self.checkpoints_mock = cm.data_protection.checkpoints
        self.checkpoints_mock.reset_mock()


class TestListCheckpoints(TestCheckpoints):
    def setUp(self):
        super(TestListCheckpoints, self).setUp()

        # Command to test
        self.cmd = osc_checkpoints.ListCheckpoints(self.app, None)

    def test_checkpoints_list(self):
        self.checkpoints_mock.list.return_value = [checkpoints.Checkpoint(
            None, copy.deepcopy(CHECKPOINT_INFO))]
        arglist = ['cf56bd3e-97a7-4078-b6d5-f36246333fd9']
        verifylist = [('provider_id', 'cf56bd3e-97a7-4078-b6d5-f36246333fd9')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = (
            ['Id', 'Project id', 'Status', 'Protection plan', 'Metadata',
             'Created at'])
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [(
            "dcb20606-ad71-40a3-80e4-ef0fafdad0c3",
            "e486a2f49695423ca9c47e589b948108",
            "available",
            "Name: %(name)s\nId: %(id)s" % {
                "id": "3523a271-68aa-42f5-b9ba-56e5200a2ebb",
                "name": "My application",
            },
            '',
            '')]
        self.assertEqual(expected_data, list(data))

    def test_checkpoints_list_with_all_projects(self):
        self.checkpoints_mock.list.return_value = [checkpoints.Checkpoint(
            None, copy.deepcopy(CHECKPOINT_INFO)), checkpoints.Checkpoint(
            None, copy.deepcopy(CHECKPOINT_INFO_2))]
        arglist = ['cf56bd3e-97a7-4078-b6d5-f36246333fd9', '--all-projects']
        verifylist = [('provider_id', 'cf56bd3e-97a7-4078-b6d5-f36246333fd9'),
                      ('all_projects', True)]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        expected_columns = (
            ['Id', 'Project id', 'Status', 'Protection plan', 'Metadata',
             'Created at'])
        self.assertEqual(expected_columns, columns)

        expected_data = [(
            "dcb20606-ad71-40a3-80e4-ef0fafdad0c3",
            "e486a2f49695423ca9c47e589b948108",
            "available",
            "Name: %(name)s\nId: %(id)s" % {
                "id": "3523a271-68aa-42f5-b9ba-56e5200a2ebb",
                "name": "My application",
            },
            '',
            ''), (
            "a6fd95fe-0892-43b2-ad3c-e56f3a1b86b8",
            "79b35e99a6a541b3bcede40f590d6878",
            "available",
            "Name: %(name)s\nId: %(id)s" % {
                "id": "3b47fd5d-21f9-4e63-8409-0acb1bffc038",
                "name": "My application",
            },
            '',
            '')
        ]
        self.assertEqual(expected_data, list(data))


class TestCreateCheckpoint(TestCheckpoints):
    def setUp(self):
        super(TestCreateCheckpoint, self).setUp()
        self.checkpoints_mock.create.return_value = checkpoints.Checkpoint(
            None, copy.deepcopy(CHECKPOINT_INFO))
        # Command to test
        self.cmd = osc_checkpoints.CreateCheckpoint(self.app, None)

    def test_checkpoint_create(self):
        arglist = ['cf56bd3e-97a7-4078-b6d5-f36246333fd9',
                   '3523a271-68aa-42f5-b9ba-56e5200a2ebb']
        verifylist = [('provider_id', 'cf56bd3e-97a7-4078-b6d5-f36246333fd9'),
                      ('plan_id', '3523a271-68aa-42f5-b9ba-56e5200a2ebb')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.checkpoints_mock.create.assert_called_once_with(
            'cf56bd3e-97a7-4078-b6d5-f36246333fd9',
            '3523a271-68aa-42f5-b9ba-56e5200a2ebb',
            None)


class TestShowCheckpoint(TestCheckpoints):
    def setUp(self):
        super(TestShowCheckpoint, self).setUp()
        self.checkpoints_mock.get.return_value = checkpoints.Checkpoint(
            None, copy.deepcopy(CHECKPOINT_INFO))
        # Command to test
        self.cmd = osc_checkpoints.ShowCheckpoint(self.app, None)

    def test_checkpoint_show(self):
        arglist = ['cf56bd3e-97a7-4078-b6d5-f36246333fd9',
                   'dcb20606-ad71-40a3-80e4-ef0fafdad0c3']
        verifylist = [('provider_id', 'cf56bd3e-97a7-4078-b6d5-f36246333fd9'),
                      ('checkpoint_id',
                       'dcb20606-ad71-40a3-80e4-ef0fafdad0c3')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.checkpoints_mock.get.assert_called_once_with(
            'cf56bd3e-97a7-4078-b6d5-f36246333fd9',
            'dcb20606-ad71-40a3-80e4-ef0fafdad0c3')


class TestDeleteCheckpoint(TestCheckpoints):
    def setUp(self):
        super(TestDeleteCheckpoint, self).setUp()
        self.checkpoints_mock.get.return_value = checkpoints.Checkpoint(
            None, copy.deepcopy(CHECKPOINT_INFO))
        # Command to test
        self.cmd = osc_checkpoints.DeleteCheckpoint(self.app, None)

    def test_checkpoint_delete(self):
        arglist = ['cf56bd3e-97a7-4078-b6d5-f36246333fd9',
                   'dcb20606-ad71-40a3-80e4-ef0fafdad0c3']
        verifylist = [('provider_id', 'cf56bd3e-97a7-4078-b6d5-f36246333fd9'),
                      ('checkpoint',
                       ['dcb20606-ad71-40a3-80e4-ef0fafdad0c3'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.checkpoints_mock.delete.assert_called_once_with(
            'cf56bd3e-97a7-4078-b6d5-f36246333fd9',
            'dcb20606-ad71-40a3-80e4-ef0fafdad0c3')


class TestResetCheckpointState(TestCheckpoints):
    def setUp(self):
        super(TestResetCheckpointState, self).setUp()
        self.cmd = osc_checkpoints.ResetCheckpointState(self.app, None)

    def test_reset_checkpoint_with_default_state(self):
        arglist = ['cf56bd3e-97a7-4078-b6d5-f36246333fd9',
                   'dcb20606-ad71-40a3-80e4-ef0fafdad0c3']
        verifylist = [('provider_id', 'cf56bd3e-97a7-4078-b6d5-f36246333fd9'),
                      ('checkpoint',
                       ['dcb20606-ad71-40a3-80e4-ef0fafdad0c3']),
                      ('state', 'error')]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.checkpoints_mock.reset_state.assert_called_once_with(
            'cf56bd3e-97a7-4078-b6d5-f36246333fd9',
            'dcb20606-ad71-40a3-80e4-ef0fafdad0c3',
            'error')

    def test_reset_checkpoint(self):
        arglist = ['cf56bd3e-97a7-4078-b6d5-f36246333fd9',
                   'dcb20606-ad71-40a3-80e4-ef0fafdad0c3',
                   '--available']
        verifylist = [('provider_id', 'cf56bd3e-97a7-4078-b6d5-f36246333fd9'),
                      ('checkpoint',
                       ['dcb20606-ad71-40a3-80e4-ef0fafdad0c3']),
                      ('state', 'available')]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.checkpoints_mock.reset_state.assert_called_once_with(
            'cf56bd3e-97a7-4078-b6d5-f36246333fd9',
            'dcb20606-ad71-40a3-80e4-ef0fafdad0c3',
            'available')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from karborclient.tests.unit import base
from karborclient.tests.unit.v1 import fakes

cs = fakes.FakeClient()
mock_request_return = ({}, {'checkpoint': {}})

FAKE_PROVIDER_ID = "2220f8b1-975d-4621-a872-fa9afb43cb6c"
FAKE_PLAN_ID = "3330f8b1-975d-4621-a872-fa9afb43cb6c"
FAKE_CHECKPOINT_ID = "e4381b1a-905e-4fec-8104-b4419ccaf963"


class CheckpointsTest(base.TestCaseShell):

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_checkpoints(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.checkpoints.list(provider_id=FAKE_PROVIDER_ID)
        mock_request.assert_called_with(
            'GET',
            '/providers/{provider_id}/checkpoints'.format(
                provider_id=FAKE_PROVIDER_ID), headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_checkpoints_with_all_tenants(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.checkpoints.list(provider_id=FAKE_PROVIDER_ID,
                            search_opts={'all_tenants': 1})
        mock_request.assert_called_with(
            'GET',
            '/providers/{provider_id}/checkpoints?all_tenants=1'.format(
                provider_id=FAKE_PROVIDER_ID), headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_get_checkpoint(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.checkpoints.get(FAKE_PROVIDER_ID, '1')
        mock_request.assert_called_with(
            'GET',
            '/providers/{provider_id}/checkpoints/1'.format(
                provider_id=FAKE_PROVIDER_ID), headers={})

    @mock.patch('karborclient.common.http.HTTPClient.raw_request')
    def test_delete_checkpoint(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.checkpoints.delete(FAKE_PROVIDER_ID, '1')
        mock_request.assert_called_with(
            'DELETE',
            '/providers/{provider_id}/checkpoints/1'.format(
                provider_id=FAKE_PROVIDER_ID), headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_checkpoints_with_marker_limit(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.checkpoints.list(provider_id=FAKE_PROVIDER_ID,
                            marker=1234, limit=2)
        mock_request.assert_called_with(
            'GET',
            '/providers/{provider_id}/'
            'checkpoints?limit=2&marker=1234'.format(
                provider_id=FAKE_PROVIDER_ID), headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_checkpoints_with_sort_key_dir(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.checkpoints.list(provider_id=FAKE_PROVIDER_ID,
                            sort_key='id', sort_dir='asc')
        mock_request.assert_called_with(
            'GET',
            '/providers/{provider_id}/'
            'checkpoints?sort_dir=asc&sort_key=id'.format(
                provider_id=FAKE_PROVIDER_ID), headers={})

    def test_list_checkpoints_with_invalid_sort_key(self):
        self.assertRaises(ValueError,
                          cs.checkpoints.list, FAKE_PROVIDER_ID,
                          sort_key='invalid', sort_dir='asc')

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_create_checkpoint(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.checkpoints.create(FAKE_PROVIDER_ID, FAKE_PLAN_ID)
        mock_request.assert_called_with(
            'POST',
            '/providers/{provider_id}/'
            'checkpoints'.format(
                provider_id=FAKE_PROVIDER_ID),
            data={
                'checkpoint': {'plan_id': FAKE_PLAN_ID, 'extra-info': None}},
            headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_reset_checkpoint_state(self, mock_request):
        mock_request.return_value = ({}, {})
        cs.checkpoints.reset_state(
            FAKE_PROVIDER_ID, FAKE_CHECKPOINT_ID, 'error')
        mock_request.assert_called_with(
            'PUT',
            '/providers/{provider_id}/checkpoints/{checkpoint_id}'.format(
                provider_id=FAKE_PROVIDER_ID,
                checkpoint_id=FAKE_CHECKPOINT_ID
            ),
            data={'os-resetState': {'state': 'error'}},
            headers={})



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from karborclient.tests.unit import base
from karborclient.tests.unit.v1 import fakes

cs = fakes.FakeClient()
mock_request_return = ({}, {'scheduled_operation': {'name': 'fake_name'}})


class ScheduledOperationsTest(base.TestCaseShell):

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_scheduled_operations_with_marker_limit(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.scheduled_operations.list(marker=1234, limit=2)
        mock_request.assert_called_with(
            'GET',
            '/scheduled_operations?limit=2&marker=1234', headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list__scheduled_operations_with_sort_key_dir(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.scheduled_operations.list(sort_key='id', sort_dir='asc')
        mock_request.assert_called_with(
            'GET',
            '/scheduled_operations?'
            'sort_dir=asc&sort_key=id', headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_scheduled_operations_with_invalid_sort_key(self,
                                                             mock_request):
        self.assertRaises(ValueError,
                          cs.scheduled_operations.list, sort_key='invalid',
                          sort_dir='asc')

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_create_scheduled_operation(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.scheduled_operations.create(
            'name', 'operation_type',
            'efc6a88b-9096-4bb6-8634-cda182a6e12a',
            'operation_definition')
        mock_request.assert_called_with(
            'POST',
            '/scheduled_operations',
            data={
                'scheduled_operation': {
                    'name': 'name',
                    'operation_type': 'operation_type',
                    'trigger_id': 'efc6a88b-9096-4bb6-8634-cda182a6e12a',
                    'operation_definition': 'operation_definition'}},
            headers={})

    @mock.patch('karborclient.common.http.HTTPClient.raw_request')
    def test_delete_scheduled_operation(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.scheduled_operations.delete('1')
        mock_request.assert_called_with(
            'DELETE',
            '/scheduled_operations/1',
            headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_show_scheduled_operation(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.scheduled_operations.get('1')
        mock_request.assert_called_with(
            'GET',
            '/scheduled_operations/1',
            headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_show_scheduled_operation_with_headers(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.scheduled_operations.get('1', session_id='fake_session_id')
        mock_request.assert_called_with(
            'GET',
            '/scheduled_operations/1',
            headers={'X-Configuration-Session': 'fake_session_id'})



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from karborclient.osc.v1 import quotas as osc_quotas
from karborclient.tests.unit.osc.v1 import fakes
from karborclient.v1 import quotas


QUOTA_INFO = {
    "id": "73f74f90a1754bd7ad658afb3272323f",
    "plans": "40"
}


class TestQuotas(fakes.TestDataProtection):
    def setUp(self):
        super(TestQuotas, self).setUp()
        self.quotas_mock = self.app.client_manager.data_protection.quotas
        self.quotas_mock.reset_mock()


class TestUpdateQuotas(TestQuotas):
    def setUp(self):
        super(TestUpdateQuotas, self).setUp()
        self.quotas_mock.update.return_value = quotas.Quota(
            None, copy.deepcopy(QUOTA_INFO))
        self.cmd = osc_quotas.UpdateQuotas(self.app, None)

    def test_quotas_update(self):
        arglist = ['--plans',
                   '40', '73f74f90a1754bd7ad658afb3272323f']
        verifylist = [('plans', 40),
                      ('tenant',
                       '73f74f90a1754bd7ad658afb3272323f')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        self.quotas_mock.update.assert_called_once_with(
            '73f74f90a1754bd7ad658afb3272323f',
            {'plans': 40})


class TestShowQuotas(TestQuotas):
    def setUp(self):
        super(TestShowQuotas, self).setUp()
        self._quota_info = copy.deepcopy(QUOTA_INFO)
        self.quotas_mock.get.return_value = quotas.Quota(
            None, copy.deepcopy(QUOTA_INFO))

        self.cmd = osc_quotas.ShowQuotas(self.app, None)

    def test_quota_show(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(('id', 'plans'), columns)
        self.assertEqual(('73f74f90a1754bd7ad658afb3272323f', '40'),
                         data)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from oslo_serialization import jsonutils

from karborclient.osc.v1 import restores as osc_restores
from karborclient.tests.unit.osc.v1 import fakes
from karborclient.v1 import restores


RESTORE_INFO = {
    "id": "22b82aa7-9179-4c71-bba2-caf5c0e68db7",
    "project_id": "e486a2f49695423ca9c47e589b948108",
    "provider_id": "cf56bd3e-97a7-4078-b6d5-f36246333fd9",
    "checkpoint_id": "dcb20606-ad71-40a3-80e4-ef0fafdad0c3",
    "restore_target": "",
    "parameters": {},
    "restore_auth": {},
    "resources_status": {},
    "resources_reason": {},
    "status": "success"
}


class TestRestores(fakes.TestDataProtection):
    def setUp(self):
        super(TestRestores, self).setUp()
        self.restores_mock = self.app.client_manager.data_protection.restores
        self.restores_mock.reset_mock()


class TestListRestores(TestRestores):
    def setUp(self):
        super(TestListRestores, self).setUp()
        self.restores_mock.list.return_value = [restores.Restore(
            None, copy.deepcopy(RESTORE_INFO))]

        # Command to test
        self.cmd = osc_restores.ListRestores(self.app, None)

    def test_restores_list(self):
        arglist = ['--status', 'success']
        verifylist = [('status', 'success')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = (
            ['Id', 'Project id', 'Provider id', 'Checkpoint id',
             'Restore target', 'Parameters', 'Status'])
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        expected_data = [("22b82aa7-9179-4c71-bba2-caf5c0e68db7",
                          "e486a2f49695423ca9c47e589b948108",
                          "cf56bd3e-97a7-4078-b6d5-f36246333fd9",
                          "dcb20606-ad71-40a3-80e4-ef0fafdad0c3",
                          "",
                          jsonutils.dumps({}),
                          "success")]
        self.assertEqual(expected_data, list(data))


class TestCreateRestore(TestRestores):
    def setUp(self):
        super(TestCreateRestore, self).setUp()
        self.restores_mock.create.return_value = restores.Restore(
            None, copy.deepcopy(RESTORE_INFO))
        # Command to test
        self.cmd = osc_restores.CreateRestore(self.app, None)

    def test_restore_create(self):
        arglist = ['cf56bd3e-97a7-4078-b6d5-f36246333fd9',
                   'dcb20606-ad71-40a3-80e4-ef0fafdad0c3']
        verifylist = [('provider_id', 'cf56bd3e-97a7-4078-b6d5-f36246333fd9'),
                      ('checkpoint_id',
                       'dcb20606-ad71-40a3-80e4-ef0fafdad0c3')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.restores_mock.create.assert_called_once_with(
            'cf56bd3e-97a7-4078-b6d5-f36246333fd9',
            'dcb20606-ad71-40a3-80e4-ef0fafdad0c3',
            None, {}, None)


class TestShowRestore(TestRestores):
    def setUp(self):
        super(TestShowRestore, self).setUp()
        self._restore_info = copy.deepcopy(RESTORE_INFO)
        self.restores_mock.get.return_value = restores.Restore(
            None, self._restore_info)

        # Command to test
        self.cmd = osc_restores.ShowRestore(self.app, None)

    def test_restore_show(self):
        arglist = ['22b82aa7-9179-4c71-bba2-caf5c0e68db7']
        verifylist = [('restore', '22b82aa7-9179-4c71-bba2-caf5c0e68db7')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = (
            'checkpoint_id', 'id', 'parameters', 'project_id',
            'provider_id', 'resources_reason', 'resources_status',
            'restore_auth', 'restore_target', 'status')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        self.assertEqual(self._restore_info['checkpoint_id'], data[0])
        self.assertEqual(self._restore_info['id'], data[1])
        self.assertEqual(self._restore_info['parameters'], data[2])
        self.assertEqual(self._restore_info['project_id'], data[3])
        self.assertEqual(self._restore_info['provider_id'], data[4])
        self.assertEqual(self._restore_info['resources_reason'], data[5])
        self.assertEqual(self._restore_info['resources_status'], data[6])
        self.assertEqual(self._restore_info['restore_auth'], data[7])
        self.assertEqual(self._restore_info['restore_target'], data[8])
        self.assertEqual(self._restore_info['status'], data[9])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from karborclient.tests.unit import base
from karborclient.tests.unit.v1 import fakes

cs = fakes.FakeClient()
mock_request_return = ({}, {'quota_class': {'plans': 50}})


class QuotaClassesTest(base.TestCaseShell):

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_quota_class_update(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.quota_classes.update('default', {'plans': 50})
        mock_request.assert_called_with(
            'PUT',
            '/quota_classes/default',
            data={'quota_class': {'plans': 50}}, headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_quota_class_update_with_none(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.quota_classes.update('default', {'plans': None})
        mock_request.assert_called_with(
            'PUT',
            '/quota_classes/default',
            data={'quota_class': {'plans': 50}}, headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_show_quota_class(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.quota_classes.get('default')
        mock_request.assert_called_with(
            'GET',
            '/quota_classes/default',
            headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_show_quota_class_with_headers(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.quota_classes.get('default', session_id='fake_session_id')
        mock_request.assert_called_with(
            'GET',
            '/quota_classes/default',
            headers={'X-Configuration-Session': 'fake_session_id'})



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import argparse
import os

from datetime import datetime
from oslo_serialization import jsonutils
from oslo_utils import uuidutils

from karborclient.common.apiclient import exceptions
from karborclient.common import base
from karborclient.common import utils
from karborclient import utils as arg_utils


@utils.arg('--all-tenants',
           dest='all_tenants',
           metavar='<0|1>',
           nargs='?',
           type=int,
           const=1,
           default=0,
           help='Shows details for all tenants. Admin only.')
@utils.arg('--name',
           metavar='<name>',
           default=None,
           help='Filters results by a name. Default=None.')
@utils.arg('--description',
           metavar='<description>',
           default=None,
           help='Filters results by a description. Default=None.')
@utils.arg('--status',
           metavar='<status>',
           default=None,
           help='Filters results by a status. Default=None.')
@utils.arg('--marker',
           metavar='<marker>',
           default=None,
           help='Begin returning plans that appear later in the plan '
                'list than that represented by this plan id. '
                'Default=None.')
@utils.arg('--limit',
           metavar='<limit>',
           default=None,
           help='Maximum number of plans to return. Default=None.')
@utils.arg('--sort_key',
           metavar='<sort_key>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort_dir',
           metavar='<sort_dir>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort',
           metavar='<key>[:<direction>]',
           default=None,
           help=(('Comma-separated list of sort keys and directions in the '
                  'form of <key>[:<asc|desc>]. '
                  'Valid keys: %s. '
                  'Default=None.') % ', '.join(base.SORT_KEY_VALUES)))
@utils.arg('--tenant',
           type=str,
           dest='tenant',
           nargs='?',
           metavar='<tenant>',
           help='Display information from single tenant (Admin only).')
def do_plan_list(cs, args):
    """Lists all plans."""

    all_tenants = 1 if args.tenant else \
        int(os.environ.get("ALL_TENANTS", args.all_tenants))
    search_opts = {
        'all_tenants': all_tenants,
        'project_id': args.tenant,
        'name': args.name,
        'description': args.description,
        'status': args.status,
    }

    if args.sort and (args.sort_key or args.sort_dir):
        raise exceptions.CommandError(
            'The --sort_key and --sort_dir arguments are deprecated and are '
            'not supported with --sort.')

    plans = cs.plans.list(search_opts=search_opts, marker=args.marker,
                          limit=args.limit, sort_key=args.sort_key,
                          sort_dir=args.sort_dir, sort=args.sort)

    key_list = ['Id', 'Name', 'Description', 'Provider id', 'Status']

    if args.sort_key or args.sort_dir or args.sort:
        sortby_index = None
    else:
        sortby_index = 0
    utils.print_list(plans, key_list, exclude_unavailable=True,
                     sortby_index=sortby_index)


@utils.arg('name',
           metavar='<name>',
           help='Plan name.')
@utils.arg('provider_id',
           metavar='<provider_id>',
           help='ID of provider.')
@utils.arg('resources',
           metavar='<id=type=name=extra_info,id=type=name=extra_info>',
           help='Resource in list must be a dict when creating'
                ' a plan. The keys of resource are id ,type, name and '
                'extra_info. The extra_info field is optional.')
@utils.arg('--parameters-json',
           type=str,
           dest='parameters_json',
           metavar='<parameters>',
           default=None,
           help='Plan parameters in json format.')
@utils.arg('--parameters',
           action='append',
           metavar='resource_type=<type>[,resource_id=<id>,key=val,...]',
           default=[],
           help='Plan parameters, may be specified multiple times. '
           'resource_type: type of resource to apply parameters. '
           'resource_id: limit the parameters to a specific resource. '
           'Other keys and values: according to provider\'s protect schema.'
           )
@utils.arg('--description',
           metavar='<description>',
           help='The description of a plan.')
def do_plan_create(cs, args):
    """Creates a plan."""
    if not uuidutils.is_uuid_like(args.provider_id):
        raise exceptions.CommandError(
            "Invalid provider id provided.")
    plan_resources = arg_utils.extract_resources(args)
    arg_utils.check_resources(cs, plan_resources)
    plan_parameters = arg_utils.extract_parameters(args)
    plan = cs.plans.create(args.name, args.provider_id, plan_resources,
                           plan_parameters, description=args.description)
    dict_format_list = {"resources", "parameters"}
    utils.print_dict(plan.to_dict(), dict_format_list=dict_format_list)


@utils.arg('plan',
           metavar='<plan>',
           help='ID of plan.')
def do_plan_show(cs, args):
    """Shows plan details."""
    plan = cs.plans.get(args.plan)
    dict_format_list = {"resources", "parameters"}
    utils.print_dict(plan.to_dict(), dict_format_list=dict_format_list)


@utils.arg('plan',
           metavar='<plan>',
           nargs="+",
           help='ID of plan.')
def do_plan_delete(cs, args):
    """Deletes plan."""
    failure_count = 0
    for plan_id in args.plan:
        try:
            plan = utils.find_resource(cs.plans, plan_id)
            cs.plans.delete(plan.id)
        except exceptions.NotFound:
            failure_count += 1
            print("Failed to delete '{0}'; plan not found".
                  format(plan_id))
    if failure_count == len(args.plan):
        raise exceptions.CommandError("Unable to find and delete any of the "
                                      "specified plan.")


@utils.arg("plan_id", metavar="<PLAN ID>",
           help="Id of plan to update.")
@utils.arg("--name", metavar="<name>",
           help="A name to which the plan will be renamed.")
@utils.arg("--description", metavar="<description>",
           help="Description to which the plan will be updated.")
@utils.arg("--resources", metavar="<id=type=name,id=type=name>",
           help="Resources to which the plan will be updated.")
@utils.arg("--status", metavar="<suspended|started>",
           help="status to which the plan will be updated.")
def do_plan_update(cs, args):
    """Updatas a plan."""
    data = {}
    if args.name is not None:
        data['name'] = args.name
    if args.description is not None:
            data['description'] = args.description
    if args.resources is not None:
        plan_resources = arg_utils.extract_resources(args)
        data['resources'] = plan_resources
    if args.status is not None:
        data['status'] = args.status
    try:
        plan = utils.find_resource(cs.plans, args.plan_id)
        plan = cs.plans.update(plan.id, data)
    except exceptions.NotFound:
        raise exceptions.CommandError("Plan %s not found" % args.plan_id)
    else:
        utils.print_dict(plan.to_dict())


@utils.arg('provider_id',
           metavar='<provider_id>',
           help='Provider id.')
@utils.arg('checkpoint_id',
           metavar='<checkpoint_id>',
           help='Checkpoint id.')
@utils.arg('--restore_target',
           metavar='<restore_target>',
           help='Restore target.')
@utils.arg('--restore_username',
           metavar='<restore_username>',
           default=None,
           help='Username to restore target.')
@utils.arg('--restore_password',
           metavar='<restore_password>',
           default=None,
           help='Password to restore target.')
@utils.arg('--parameters-json',
           type=str,
           dest='parameters_json',
           metavar='<parameters>',
           default=None,
           help='Restore parameters in json format.')
@utils.arg('--parameters',
           action='append',
           metavar='resource_type=<type>[,resource_id=<id>,key=val,...]',
           default=[],
           help='Restore parameters, may be specified multiple times. '
           'resource_type: type of resource to apply parameters. '
           'resource_id: limit the parameters to a specific resource. '
           'Other keys and values: according to provider\'s restore schema.'
           )
def do_restore_create(cs, args):
    """Creates a restore."""
    if not uuidutils.is_uuid_like(args.provider_id):
        raise exceptions.CommandError(
            "Invalid provider id provided.")

    if not uuidutils.is_uuid_like(args.checkpoint_id):
        raise exceptions.CommandError(
            "Invalid checkpoint id provided.")

    restore_parameters = arg_utils.extract_parameters(args)
    restore_auth = None
    if args.restore_target is not None:
        if args.restore_username is None:
            raise exceptions.CommandError(
                "Must specify username for restore_target.")
        if args.restore_password is None:
            raise exceptions.CommandError(
                "Must specify password for restore_target.")
        restore_auth = {
            'type': 'password',
            'username': args.restore_username,
            'password': args.restore_password,
        }
    restore = cs.restores.create(args.provider_id, args.checkpoint_id,
                                 args.restore_target, restore_parameters,
                                 restore_auth)
    dict_format_list = {"parameters"}
    utils.print_dict(restore.to_dict(), dict_format_list=dict_format_list)


@utils.arg('--all-tenants',
           dest='all_tenants',
           metavar='<0|1>',
           nargs='?',
           type=int,
           const=1,
           default=0,
           help='Shows details for all tenants. Admin only.')
@utils.arg('--status',
           metavar='<status>',
           default=None,
           help='Filters results by a status. Default=None.')
@utils.arg('--marker',
           metavar='<marker>',
           default=None,
           help='Begin returning restores that appear later in the restore '
                'list than that represented by this restore id. '
                'Default=None.')
@utils.arg('--limit',
           metavar='<limit>',
           default=None,
           help='Maximum number of restores to return. Default=None.')
@utils.arg('--sort_key',
           metavar='<sort_key>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort_dir',
           metavar='<sort_dir>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort',
           metavar='<key>[:<direction>]',
           default=None,
           help=(('Comma-separated list of sort keys and directions in the '
                  'form of <key>[:<asc|desc>]. '
                  'Valid keys: %s. '
                  'Default=None.') % ', '.join(base.SORT_KEY_VALUES)))
@utils.arg('--tenant',
           type=str,
           dest='tenant',
           nargs='?',
           metavar='<tenant>',
           help='Display information from single tenant (Admin only).')
def do_restore_list(cs, args):
    """Lists all restores."""

    all_tenants = 1 if args.tenant else \
        int(os.environ.get("ALL_TENANTS", args.all_tenants))
    search_opts = {
        'all_tenants': all_tenants,
        'project_id': args.tenant,
        'status': args.status,
    }

    if args.sort and (args.sort_key or args.sort_dir):
        raise exceptions.CommandError(
            'The --sort_key and --sort_dir arguments are deprecated and are '
            'not supported with --sort.')

    restores = cs.restores.list(search_opts=search_opts, marker=args.marker,
                                limit=args.limit, sort_key=args.sort_key,
                                sort_dir=args.sort_dir, sort=args.sort)

    key_list = ['Id', 'Project id', 'Provider id', 'Checkpoint id',
                'Restore target', 'Parameters', 'Status']

    if args.sort_key or args.sort_dir or args.sort:
        sortby_index = None
    else:
        sortby_index = 0
    formatters = {"Parameters": lambda obj: jsonutils.dumps(
        obj.parameters, indent=2, sort_keys=True)}
    utils.print_list(restores, key_list, exclude_unavailable=True,
                     sortby_index=sortby_index, formatters=formatters)


@utils.arg('restore',
           metavar='<restore>',
           help='ID of restore.')
def do_restore_show(cs, args):
    """Shows restore details."""
    restore = cs.restores.get(args.restore)
    dict_format_list = {"parameters"}
    utils.print_dict(restore.to_dict(), dict_format_list=dict_format_list)


@utils.arg('provider_id',
           metavar='<provider_id>',
           help='Provider id.')
@utils.arg('checkpoint_id',
           metavar='<checkpoint_id>',
           help='Checkpoint id.')
@utils.arg('--parameters-json',
           type=str,
           dest='parameters_json',
           metavar='<parameters>',
           default=None,
           help='Verification parameters in json format.')
@utils.arg('--parameters',
           action='append',
           metavar='resource_type=<type>[,resource_id=<id>,key=val,...]',
           default=[],
           help='Verification parameters, may be specified multiple times. '
           'resource_type: type of resource to apply parameters. '
           'resource_id: limit the parameters to a specific resource. '
           'Other keys and values: according to provider\'s schema.'
           )
def do_verification_create(cs, args):
    """Creates a verification."""
    if not uuidutils.is_uuid_like(args.provider_id):
        raise exceptions.CommandError(
            "Invalid provider id provided.")

    if not uuidutils.is_uuid_like(args.checkpoint_id):
        raise exceptions.CommandError(
            "Invalid checkpoint id provided.")

    verification_parameters = arg_utils.extract_parameters(args)
    verification = cs.verifications.create(args.provider_id,
                                           args.checkpoint_id,
                                           verification_parameters)
    dict_format_list = {"parameters"}
    utils.print_dict(verification.to_dict(), dict_format_list=dict_format_list)


@utils.arg('--all-tenants',
           dest='all_tenants',
           metavar='<0|1>',
           nargs='?',
           type=int,
           const=1,
           default=0,
           help='Shows details for all tenants. Admin only.')
@utils.arg('--status',
           metavar='<status>',
           default=None,
           help='Filters results by a status. Default=None.')
@utils.arg('--marker',
           metavar='<marker>',
           default=None,
           help='Begin returning verifications that appear later in the'
                'list than that represented by this verification id. '
                'Default=None.')
@utils.arg('--limit',
           metavar='<limit>',
           default=None,
           help='Maximum number of verifications to return. Default=None.')
@utils.arg('--sort_key',
           metavar='<sort_key>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort_dir',
           metavar='<sort_dir>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort',
           metavar='<key>[:<direction>]',
           default=None,
           help=(('Comma-separated list of sort keys and directions in the '
                  'form of <key>[:<asc|desc>]. '
                  'Valid keys: %s. '
                  'Default=None.') % ', '.join(base.SORT_KEY_VALUES)))
@utils.arg('--tenant',
           type=str,
           dest='tenant',
           nargs='?',
           metavar='<tenant>',
           help='Display information from single tenant (Admin only).')
def do_verification_list(cs, args):
    """Lists all verifications."""

    all_tenants = 1 if args.tenant else \
        int(os.environ.get("ALL_TENANTS", args.all_tenants))
    search_opts = {
        'all_tenants': all_tenants,
        'project_id': args.tenant,
        'status': args.status,
    }

    if args.sort and (args.sort_key or args.sort_dir):
        raise exceptions.CommandError(
            'The --sort_key and --sort_dir arguments are '
            'not supported with --sort.')

    verifications = cs.verifications.list(search_opts=search_opts,
                                          marker=args.marker,
                                          limit=args.limit,
                                          sort_key=args.sort_key,
                                          sort_dir=args.sort_dir,
                                          sort=args.sort)

    key_list = ['Id', 'Project id', 'Provider id', 'Checkpoint id',
                'Parameters', 'Status']

    if args.sort_key or args.sort_dir or args.sort:
        sortby_index = None
    else:
        sortby_index = 0
    formatters = {"Parameters": lambda obj: jsonutils.dumps(
        obj.parameters, indent=2, sort_keys=True)}
    utils.print_list(verifications, key_list, exclude_unavailable=True,
                     sortby_index=sortby_index, formatters=formatters)


@utils.arg('verification',
           metavar='<verification>',
           help='ID of verification.')
def do_verification_show(cs, args):
    """Shows verification details."""
    verification = cs.verifications.get(args.verification)
    dict_format_list = {"parameters"}
    utils.print_dict(verification.to_dict(),
                     dict_format_list=dict_format_list)


def do_protectable_list(cs, args):
    """Lists all protectable types."""

    protectables = cs.protectables.list()

    key_list = ['Protectable type']

    utils.print_list(protectables, key_list, exclude_unavailable=True)


@utils.arg('protectable_type',
           metavar='<protectable_type>',
           help='Protectable type.')
def do_protectable_show(cs, args):
    """Shows protectable type details."""
    protectable = cs.protectables.get(args.protectable_type)
    utils.print_dict(protectable.to_dict())


@utils.arg('protectable_type',
           metavar='<protectable_type>',
           help='Protectable type.')
@utils.arg('protectable_id',
           metavar='<protectable_id>',
           help='Protectable instance id.')
@utils.arg('--parameters',
           type=str,
           nargs='*',
           metavar='<key=value>',
           default=None,
           help='Show a instance by parameters key and value pair. '
                'Default=None.')
def do_protectable_show_instance(cs, args):
    """Shows instance details."""
    search_opts = {
        'parameters': (arg_utils.extract_instances_parameters(args)
                       if args.parameters else None),
    }
    instance = cs.protectables.get_instance(args.protectable_type,
                                            args.protectable_id,
                                            search_opts=search_opts)
    dict_format_list = {"dependent_resources"}
    utils.print_dict(instance.to_dict(), dict_format_list=dict_format_list)


@utils.arg('protectable_type',
           metavar='<protectable_type>',
           help='Type of protectable.')
@utils.arg('--type',
           metavar='<type>',
           default=None,
           help='Filters results by a status. Default=None.')
@utils.arg('--marker',
           metavar='<marker>',
           default=None,
           help='Begin returning instances that appear later in the instance '
                'list than that represented by this instance id. '
                'Default=None.')
@utils.arg('--limit',
           metavar='<limit>',
           default=None,
           help='Maximum number of instances to return. Default=None.')
@utils.arg('--sort_key',
           metavar='<sort_key>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort_dir',
           metavar='<sort_dir>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort',
           metavar='<key>[:<direction>]',
           default=None,
           help=(('Comma-separated list of sort keys and directions in the '
                  'form of <key>[:<asc|desc>]. '
                  'Valid keys: %s. '
                  'Default=None.') % ', '.join(base.SORT_KEY_VALUES)))
@utils.arg('--parameters',
           type=str,
           nargs='*',
           metavar='<key=value>',
           default=None,
           help='List instances by parameters key and value pair. '
                'Default=None.')
def do_protectable_list_instances(cs, args):
    """Lists all protectable instances."""

    search_opts = {
        'type': args.type,
        'parameters': (arg_utils.extract_instances_parameters(args)
                       if args.parameters else None),
    }

    if args.sort and (args.sort_key or args.sort_dir):
        raise exceptions.CommandError(
            'The --sort_key and --sort_dir arguments are deprecated and are '
            'not supported with --sort.')

    instances = cs.protectables.list_instances(
        args.protectable_type, search_opts=search_opts,
        marker=args.marker, limit=args.limit,
        sort_key=args.sort_key,
        sort_dir=args.sort_dir, sort=args.sort)

    key_list = ['Id', 'Type', 'Name', 'Dependent resources', 'Extra info']

    if args.sort_key or args.sort_dir or args.sort:
        sortby_index = None
    else:
        sortby_index = 0

    formatters = {"Dependent resources": lambda obj: jsonutils.dumps(
        obj.dependent_resources, indent=2, sort_keys=True)}
    utils.print_list(instances, key_list, exclude_unavailable=True,
                     sortby_index=sortby_index, formatters=formatters)


@utils.arg('provider_id',
           metavar='<provider_id>',
           help='Id of provider.')
def do_provider_show(cs, args):
    """Shows provider details."""
    provider = cs.providers.get(args.provider_id)
    dict_format_list = {"extended_info_schema"}
    utils.print_dict(provider.to_dict(), dict_format_list=dict_format_list)


@utils.arg('--name',
           metavar='<name>',
           default=None,
           help='Filters results by a name. Default=None.')
@utils.arg('--description',
           metavar='<description>',
           default=None,
           help='Filters results by a description. Default=None.')
@utils.arg('--marker',
           metavar='<marker>',
           default=None,
           help='Begin returning providers that appear later in the provider '
                'list than that represented by this provider id. '
                'Default=None.')
@utils.arg('--limit',
           metavar='<limit>',
           default=None,
           help='Maximum number of providers to return. Default=None.')
@utils.arg('--sort_key',
           metavar='<sort_key>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort_dir',
           metavar='<sort_dir>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort',
           metavar='<key>[:<direction>]',
           default=None,
           help=(('Comma-separated list of sort keys and directions in the '
                  'form of <key>[:<asc|desc>]. '
                  'Valid keys: %s. '
                  'Default=None.') % ', '.join(base.SORT_KEY_VALUES)))
def do_provider_list(cs, args):
    """Lists all providers."""

    search_opts = {
        'name': args.name,
        'description': args.description,
    }

    if args.sort and (args.sort_key or args.sort_dir):
        raise exceptions.CommandError(
            'The --sort_key and --sort_dir arguments are deprecated and are '
            'not supported with --sort.')

    providers = cs.providers.list(search_opts=search_opts, marker=args.marker,
                                  limit=args.limit, sort_key=args.sort_key,
                                  sort_dir=args.sort_dir, sort=args.sort)

    key_list = ['Id', 'Name', 'Description']

    if args.sort_key or args.sort_dir or args.sort:
        sortby_index = None
    else:
        sortby_index = 0
    utils.print_list(providers, key_list, exclude_unavailable=True,
                     sortby_index=sortby_index)


@utils.arg('provider_id',
           metavar='<provider_id>',
           help='ID of provider.')
@utils.arg('plan_id',
           metavar='<plan_id>',
           help='ID of plan.')
@utils.arg('--extra_info',
           type=str,
           nargs='*',
           metavar='<key=value>',
           default=None,
           help='The extra info of a checkpoint.')
def do_checkpoint_create(cs, args):
    """Creates a checkpoint."""

    checkpoint_extra_info = None
    if args.extra_info is not None:
        checkpoint_extra_info = arg_utils.extract_extra_info(args)
    checkpoint = cs.checkpoints.create(args.provider_id, args.plan_id,
                                       checkpoint_extra_info)
    dict_format_list = {"protection_plan"}
    json_format_list = {"resource_graph"}
    utils.print_dict(checkpoint.to_dict(), dict_format_list=dict_format_list,
                     json_format_list=json_format_list)


@utils.arg('--all-tenants',
           dest='all_tenants',
           metavar='<0|1>',
           nargs='?',
           type=int,
           const=1,
           default=0,
           help='Shows details for all tenants. Admin only.')
@utils.arg('provider_id',
           metavar='<provider_id>',
           help='ID of provider.')
@utils.arg('--plan_id',
           metavar='<plan_id>',
           default=None,
           help='Filters results by a plan ID. Default=None.')
@utils.arg('--start_date',
           type=str,
           metavar='<start_date>',
           default=None,
           help='Filters results by a start date("Y-m-d"). Default=None.')
@utils.arg('--end_date',
           type=str,
           metavar='<end_date>',
           default=None,
           help='Filters results by a end date("Y-m-d"). Default=None.')
@utils.arg('--project_id',
           metavar='<project_id>',
           default=None,
           help='Filters results by a project ID. Default=None.')
@utils.arg('--marker',
           metavar='<marker>',
           default=None,
           help='Begin returning checkpoints that appear later in the '
                'checkpoint list than that represented by this checkpoint id. '
                'Default=None.')
@utils.arg('--limit',
           metavar='<limit>',
           default=None,
           help='Maximum number of checkpoints to return. Default=None.')
@utils.arg('--sort_key',
           metavar='<sort_key>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort_dir',
           metavar='<sort_dir>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort',
           metavar='<key>[:<direction>]',
           default=None,
           help=(('Comma-separated list of sort keys and directions in the '
                  'form of <key>[:<asc|desc>]. '
                  'Valid keys: %s. '
                  'Default=None.') % ', '.join(base.SORT_KEY_VALUES)))
def do_checkpoint_list(cs, args):
    """Lists all checkpoints."""
    if args.plan_id is not None:
        if not uuidutils.is_uuid_like(args.plan_id):
            raise exceptions.CommandError('The plan_id must be a uuid')

    if args.start_date:
        try:
            datetime.strptime(
                args.start_date, "%Y-%m-%d")
        except (ValueError, SyntaxError):
            raise exceptions.CommandError(
                "The format of start_date should be %Y-%m-%d")

    if args.end_date:
        try:
            datetime.strptime(
                args.end_date, "%Y-%m-%d")
        except (ValueError, SyntaxError):
            raise exceptions.CommandError(
                "The format of end_date should be %Y-%m-%d")

    search_opts = {
        'plan_id': args.plan_id,
        'start_date': args.start_date,
        'end_date': args.end_date,
        'project_id': args.project_id,
        'all_tenants': args.all_tenants
    }

    if args.sort and (args.sort_key or args.sort_dir):
        raise exceptions.CommandError(
            'The --sort_key and --sort_dir arguments are deprecated and are '
            'not supported with --sort.')

    checkpoints = cs.checkpoints.list(
        provider_id=args.provider_id, search_opts=search_opts,
        marker=args.marker, limit=args.limit, sort_key=args.sort_key,
        sort_dir=args.sort_dir, sort=args.sort)

    key_list = ['Id', 'Project id', 'Status', 'Protection plan', 'Metadata',
                'Created at']

    if args.sort_key or args.sort_dir or args.sort:
        sortby_index = None
    else:
        sortby_index = 0

    def plan_formatter(obj):
        return "Name: %s\nId: %s" % (obj.protection_plan['name'],
                                     obj.protection_plan['id'])
    formatters = {"Protection plan": plan_formatter}
    utils.print_list(checkpoints, key_list, exclude_unavailable=True,
                     sortby_index=sortby_index, formatters=formatters)


@utils.arg('provider_id',
           metavar='<provider_id>',
           help='Id of provider.')
@utils.arg('checkpoint_id',
           metavar='<checkpoint_id>',
           help='Id of checkpoint.')
def do_checkpoint_show(cs, args):
    """Shows checkpoint details."""
    checkpoint = cs.checkpoints.get(args.provider_id, args.checkpoint_id)
    dict_format_list = {"protection_plan"}
    json_format_list = {"resource_graph"}
    utils.print_dict(checkpoint.to_dict(), dict_format_list=dict_format_list,
                     json_format_list=json_format_list)


@utils.arg('provider_id',
           metavar='<provider_id>',
           help='Id of provider.')
@utils.arg('checkpoint',
           metavar='<checkpoint>',
           nargs="+",
           help='ID of checkpoint.')
def do_checkpoint_delete(cs, args):
    """Deletes checkpoints."""
    failure_count = 0
    for checkpoint_id in args.checkpoint:
        try:
            checkpoint = cs.checkpoints.get(args.provider_id,
                                            checkpoint_id)
            cs.checkpoints.delete(args.provider_id, checkpoint.id)
        except exceptions.NotFound:
            failure_count += 1
            print("Failed to delete '{0}'; checkpoint not found".
                  format(checkpoint_id))
    if failure_count == len(args.checkpoint):
        raise exceptions.CommandError("Unable to find and delete any of the "
                                      "specified checkpoint.")


@utils.arg('provider_id',
           metavar='<provider_id>',
           help='Id of provider.')
@utils.arg('checkpoint',
           metavar='<checkpoint>',
           nargs="+",
           help='ID of checkpoint.')
@utils.arg('--available',
           action='store_const',
           dest='state',
           default='error',
           const='available',
           help='Request the checkpoint be reset to "available" state instead '
                'of "error" state(the default).')
def do_checkpoint_reset_state(cs, args):
    """Reset state of a checkpoint."""
    failure_count = 0

    for checkpoint_id in args.checkpoint:
        try:
            cs.checkpoints.reset_state(args.provider_id, checkpoint_id,
                                       args.state)
        except exceptions.NotFound:
            failure_count += 1
            print("Failed to reset state of '{0}'; checkpoint not found".
                  format(checkpoint_id))
        except exceptions.Forbidden:
            failure_count += 1
            print("Failed to reset state of '{0}'; not allowed".
                  format(checkpoint_id))
        except exceptions.BadRequest:
            failure_count += 1
            print("Failed to reset state of '{0}'; invalid input or "
                  "current checkpoint state".format(checkpoint_id))
    if failure_count == len(args.checkpoint):
        raise exceptions.CommandError("Unable to find or reset any of the "
                                      "specified checkpoint's state.")


@utils.arg('--all-tenants',
           dest='all_tenants',
           metavar='<0|1>',
           nargs='?',
           type=int,
           const=1,
           default=0,
           help='Shows details for all tenants. Admin only.')
@utils.arg('--name',
           metavar='<name>',
           default=None,
           help='Filters results by a name. Default=None.')
@utils.arg('--type',
           metavar='<type>',
           default=None,
           help='Filters results by a type. Default=None.')
@utils.arg('--properties',
           metavar='<properties>',
           default=None,
           help='Filters results by a properties. Default=None.')
@utils.arg('--marker',
           metavar='<marker>',
           default=None,
           help='Begin returning triggers that appear later in the trigger '
                'list than that represented by this trigger id. '
                'Default=None.')
@utils.arg('--limit',
           metavar='<limit>',
           default=None,
           help='Maximum number of triggers to return. Default=None.')
@utils.arg('--sort_key',
           metavar='<sort_key>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort_dir',
           metavar='<sort_dir>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort',
           metavar='<key>[:<direction>]',
           default=None,
           help=(('Comma-separated list of sort keys and directions in the '
                  'form of <key>[:<asc|desc>]. '
                  'Valid keys: %s. '
                  'Default=None.') % ', '.join(base.SORT_KEY_VALUES)))
@utils.arg('--tenant',
           type=str,
           dest='tenant',
           nargs='?',
           metavar='<tenant>',
           help='Display information from single tenant (Admin only).')
def do_trigger_list(cs, args):
    """Lists all triggers."""

    all_tenants = 1 if args.tenant else \
        int(os.environ.get("ALL_TENANTS", args.all_tenants))
    search_opts = {
        'all_tenants': all_tenants,
        'project_id': args.tenant,
        'name': args.name,
        'type': args.type,
        'properties': args.properties,
    }

    if args.sort and (args.sort_key or args.sort_dir):
        raise exceptions.CommandError(
            'The --sort_key and --sort_dir arguments are deprecated and are '
            'not supported with --sort.')

    triggers = cs.triggers.list(search_opts=search_opts, marker=args.marker,
                                limit=args.limit, sort_key=args.sort_key,
                                sort_dir=args.sort_dir, sort=args.sort)

    key_list = ['Id', 'Name', 'Type', 'Properties']

    if args.sort_key or args.sort_dir or args.sort:
        sortby_index = None
    else:
        sortby_index = 0

    formatters = {"Properties": lambda obj: jsonutils.dumps(
        obj.properties, indent=2, sort_keys=True)}
    utils.print_list(triggers, key_list, exclude_unavailable=True,
                     sortby_index=sortby_index, formatters=formatters)


@utils.arg('name',
           metavar='<name>',
           help='Trigger name.')
@utils.arg('type',
           metavar='<type>',
           help='Type of trigger.')
@utils.arg('properties',
           metavar='<key=value,key=value>',
           help='Properties of trigger.')
def do_trigger_create(cs, args):
    """Creates a trigger."""
    trigger_properties = arg_utils.extract_properties(args)
    trigger = cs.triggers.create(args.name, args.type, trigger_properties)
    dict_format_list = {"properties"}
    utils.print_dict(trigger.to_dict(), dict_format_list=dict_format_list)


@utils.arg("trigger_id", metavar="<TRIGGER ID>",
           help="Id of trigger to update.")
@utils.arg("--name", metavar="<name>",
           help="A new name to which the trigger will be renamed.")
@utils.arg("--properties", metavar="<key=value,key=value>",
           help="Properties of trigger which will be updated.")
def do_trigger_update(cs, args):
    """Update a trigger."""
    trigger_info = {}
    trigger_properties = arg_utils.extract_properties(args)
    if args.name:
        trigger_info['name'] = args.name
    trigger_info['properties'] = trigger_properties
    trigger = cs.triggers.update(args.trigger_id, trigger_info)
    dict_format_list = {"properties"}
    utils.print_dict(trigger.to_dict(), dict_format_list=dict_format_list)


@utils.arg('trigger',
           metavar='<trigger>',
           help='ID of trigger.')
def do_trigger_show(cs, args):
    """Shows trigger details."""
    trigger = cs.triggers.get(args.trigger)
    dict_format_list = {"properties"}
    utils.print_dict(trigger.to_dict(), dict_format_list=dict_format_list)


@utils.arg('trigger',
           metavar='<trigger>',
           nargs="+",
           help='ID of trigger.')
def do_trigger_delete(cs, args):
    """Deletes trigger."""
    failure_count = 0
    for trigger_id in args.trigger:
        try:
            trigger = utils.find_resource(cs.triggers, trigger_id)
            cs.triggers.delete(trigger.id)
        except exceptions.NotFound:
            failure_count += 1
            print("Failed to delete '{0}'; trigger not found".
                  format(trigger_id))
    if failure_count == len(args.trigger):
        raise exceptions.CommandError("Unable to find and delete any of the "
                                      "specified trigger.")


@utils.arg('--all-tenants',
           dest='all_tenants',
           metavar='<0|1>',
           nargs='?',
           type=int,
           const=1,
           default=0,
           help='Shows details for all tenants. Admin only.')
@utils.arg('--name',
           metavar='<name>',
           default=None,
           help='Filters results by a name. Default=None.')
@utils.arg('--operation_type',
           metavar='<operation_type>',
           default=None,
           help='Filters results by a type. Default=None.')
@utils.arg('--trigger_id',
           metavar='<trigger_id>',
           default=None,
           help='Filters results by a trigger id. Default=None.')
@utils.arg('--operation_definition',
           metavar='<operation_definition>',
           default=None,
           help='Filters results by a operation definition. Default=None.')
@utils.arg('--marker',
           metavar='<marker>',
           default=None,
           help='Begin returning resources that appear later in the '
                'list than that represented by this id. '
                'Default=None.')
@utils.arg('--limit',
           metavar='<limit>',
           default=None,
           help='Maximum number to return. Default=None.')
@utils.arg('--sort_key',
           metavar='<sort_key>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort_dir',
           metavar='<sort_dir>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort',
           metavar='<key>[:<direction>]',
           default=None,
           help=(('Comma-separated list of sort keys and directions in the '
                  'form of <key>[:<asc|desc>]. '
                  'Valid keys: %s. '
                  'Default=None.') % ', '.join(base.SORT_KEY_VALUES)))
@utils.arg('--tenant',
           type=str,
           dest='tenant',
           nargs='?',
           metavar='<tenant>',
           help='Display information from single tenant (Admin only).')
def do_scheduledoperation_list(cs, args):
    """Lists all scheduledoperations."""

    all_tenants = 1 if args.tenant else \
        int(os.environ.get("ALL_TENANTS", args.all_tenants))
    search_opts = {
        'all_tenants': all_tenants,
        'project_id': args.tenant,
        'name': args.name,
        'operation_type': args.operation_type,
        'trigger_id': args.trigger_id,
        'operation_definition': args.operation_definition,
    }

    if args.sort and (args.sort_key or args.sort_dir):
        raise exceptions.CommandError(
            'The --sort_key and --sort_dir arguments are deprecated and are '
            'not supported with --sort.')

    scheduledoperations = cs.scheduled_operations.list(
        search_opts=search_opts, marker=args.marker, limit=args.limit,
        sort_key=args.sort_key, sort_dir=args.sort_dir, sort=args.sort)

    key_list = ['Id', 'Name', 'OperationType', 'TriggerId',
                'OperationDefinition']

    if args.sort_key or args.sort_dir or args.sort:
        sortby_index = None
    else:
        sortby_index = 0
    utils.print_list(scheduledoperations, key_list, exclude_unavailable=True,
                     sortby_index=sortby_index)


@utils.arg('name',
           metavar='<name>',
           help='Trigger name.')
@utils.arg('operation_type',
           metavar='<operation_type>',
           choices=['protect', 'retention_protect'],
           help='Operation Type of scheduled operation. Valid values are '
                '"protect" or "retention_protect."')
@utils.arg('trigger_id',
           metavar='<trigger_id>',
           help='Trigger id of scheduled operation.')
@utils.arg('operation_definition',
           metavar='<key=value,key=value>',
           help='Operation definition of scheduled operation.')
def do_scheduledoperation_create(cs, args):
    """Creates a scheduled operation."""
    operation_definition = arg_utils.extract_operation_definition(args)
    scheduledoperation = cs.scheduled_operations.create(args.name,
                                                        args.operation_type,
                                                        args.trigger_id,
                                                        operation_definition)
    dict_format_list = {"operation_definition"}
    utils.print_dict(scheduledoperation.to_dict(),
                     dict_format_list=dict_format_list)


@utils.arg('scheduledoperation',
           metavar='<scheduledoperation>',
           help='ID of scheduled operation.')
def do_scheduledoperation_show(cs, args):
    """Shows scheduledoperation details."""
    scheduledoperation = cs.scheduled_operations.get(args.scheduledoperation)
    dict_format_list = {"operation_definition"}
    utils.print_dict(scheduledoperation.to_dict(),
                     dict_format_list=dict_format_list)


@utils.arg('scheduledoperation',
           metavar='<scheduledoperation>',
           nargs="+",
           help='ID of scheduled operation.')
def do_scheduledoperation_delete(cs, args):
    """Deletes a scheduled operation."""
    failure_count = 0
    for scheduledoperation_id in args.scheduledoperation:
        try:
            scheduledoperation = utils.find_resource(cs.scheduled_operations,
                                                     scheduledoperation_id)
            cs.scheduled_operations.delete(scheduledoperation.id)
        except exceptions.NotFound:
            failure_count += 1
            print("Failed to delete '{0}'; scheduledoperation not found".
                  format(scheduledoperation_id))
    if failure_count == len(args.scheduledoperation):
        raise exceptions.CommandError("Unable to find and delete any of the "
                                      "specified scheduled operation.")


@utils.arg('--all-tenants',
           dest='all_tenants',
           metavar='<0|1>',
           nargs='?',
           type=int,
           const=1,
           default=0,
           help='Shows details for all tenants. Admin only.')
@utils.arg('--status',
           metavar='<status>',
           default=None,
           help='Filters results by a status. Default=None.')
@utils.arg('--marker',
           metavar='<marker>',
           default=None,
           help='Begin returning restores that appear later in the '
                'operation_log list than that represented by this '
                'operation_logs id. Default=None.')
@utils.arg('--limit',
           metavar='<limit>',
           default=None,
           help='Maximum number of operation_logs to return. Default=None.')
@utils.arg('--sort_key',
           metavar='<sort_key>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort_dir',
           metavar='<sort_dir>',
           default=None,
           help=argparse.SUPPRESS)
@utils.arg('--sort',
           metavar='<key>[:<direction>]',
           default=None,
           help=(('Comma-separated list of sort keys and directions in the '
                  'form of <key>[:<asc|desc>]. '
                  'Valid keys: %s. '
                  'Default=None.') % ', '.join(base.SORT_KEY_VALUES)))
@utils.arg('--tenant',
           type=str,
           dest='tenant',
           nargs='?',
           metavar='<tenant>',
           help='Display information from single tenant (Admin only).')
def do_operationlog_list(cs, args):
    """Lists all operation_logs."""

    all_tenants = 1 if args.tenant else \
        int(os.environ.get("ALL_TENANTS", args.all_tenants))
    search_opts = {
        'all_tenants': all_tenants,
        'project_id': args.tenant,
        'status': args.status,
    }

    if args.sort and (args.sort_key or args.sort_dir):
        raise exceptions.CommandError(
            'The --sort_key and --sort_dir arguments are deprecated and are '
            'not supported with --sort.')

    operation_logs = cs.operation_logs.list(
        search_opts=search_opts, marker=args.marker,
        limit=args.limit, sort_key=args.sort_key,
        sort_dir=args.sort_dir, sort=args.sort)

    key_list = ['Id', 'Operation Type', 'Checkpoint id', 'Plan Id',
                'Provider id', 'Restore Id', 'Scheduled Operation Id',
                'Status', 'Started At', 'Ended At', 'Error Info', 'Extra Info']

    if args.sort_key or args.sort_dir or args.sort:
        sortby_index = None
    else:
        sortby_index = 0
    utils.print_list(operation_logs, key_list, exclude_unavailable=True,
                     sortby_index=sortby_index)


@utils.arg('operation_log',
           metavar='<operation_log>',
           help='ID of operation_log.')
def do_operationlog_show(cs, args):
    """Shows operation_log details."""
    operation_log = cs.operation_logs.get(args.operation_log)
    utils.print_dict(operation_log.to_dict())


@utils.arg('--host',
           metavar='<hostname>',
           default=None,
           help='Name of host.')
@utils.arg('--binary',
           metavar='<binary>',
           default=None,
           help='Service binary.')
def do_service_list(cs, args):
    """Show a list of all running services. Filter by host & binary."""
    result = cs.services.list(host=args.host, binary=args.binary)
    columns = ["Id", "Binary", "Host", "Status", "State",
               "Updated_at", "Disabled Reason"]
    utils.print_list(result, columns)


@utils.arg('service_id',
           metavar='<service_id>',
           help='ID of the service.')
def do_service_enable(cs, args):
    """Enable the service."""
    result = cs.services.enable(args.service_id)
    utils.print_list([result], ["Id", "Binary", "Host", "Status", "State",
                                "Updated_at", "Disabled Reason"])


@utils.arg('service_id',
           metavar='<service_id>',
           help='ID of the service.')
@utils.arg('--reason',
           metavar='<reason>',
           help='Reason for disabling the service.')
def do_service_disable(cs, args):
    """Disable the service"""
    if args.reason:
        result = cs.services.disable_log_reason(args.service_id, args.reason)
    else:
        result = cs.services.disable(args.service_id)
    utils.print_list([result], ["Id", "Binary", "Host", "Status", "State",
                                "Updated_at", "Disabled Reason"])


@utils.arg(
    '--tenant',
    metavar='<tenant>',
    default=None,
    help='ID of tenant to list the quotas for.')
@utils.arg(
    '--detail',
    action='store_true',
    help='Optional flag to indicate whether to show quota in detail. '
         'Default false.')
def do_quota_show(cs, args):
    """List the quotas for a tenant."""
    project_id = args.tenant or cs.http_client.get_project_id()
    kwargs = {
        "project_id": project_id,
        "detail": args.detail,
    }
    result = cs.quotas.get(**kwargs)
    _quota_set_pretty_show(result)


def _quota_set_pretty_show(quotas):
    """Convert quotas object to dict and display."""

    new_quotas = {}
    for quota_k, quota_v in sorted(quotas.to_dict().items()):
        if isinstance(quota_v, dict):
            quota_v = '\n'.join(
                ['%s = %s' % (k, v) for k, v in sorted(quota_v.items())])
        new_quotas[quota_k] = quota_v

    utils.print_dict(new_quotas)


@utils.arg(
    'tenant',
    metavar='<tenant>',
    help='ID of tenant to set the quotas for.')
@utils.arg(
    '--plans',
    metavar='<plans>',
    type=int,
    default=None,
    help='New value for the "plans" quota. The default value is 50.')
def do_quota_update(cs, args):
    """Update the quotas for a project (Admin only)."""
    project_id = args.tenant
    data = {
        "plans": args.plans,
    }
    result = cs.quotas.update(project_id, data)
    _quota_set_pretty_show(result)


@utils.arg(
    '--tenant',
    metavar='<tenant>',
    default=None,
    help='ID of tenant to list the quotas for.')
def do_quota_defaults(cs, args):
    """List the default quotas for a tenant."""
    project_id = args.tenant or cs.http_client.get_project_id()

    result = cs.quotas.defaults(project_id)
    _quota_set_pretty_show(result)


@utils.arg(
    'class_name',
    metavar='<class_name>',
    help='Name of quota class to list the quotas for.')
def do_quota_class_show(cs, args):
    """List the quotas for a quota class."""
    result = cs.quota_classes.get(args.class_name)
    _quota_set_pretty_show(result)


@utils.arg(
    'class_name',
    metavar='<class_name>',
    help='Name of quota class to set the quotas for.')
@utils.arg(
    '--plans',
    metavar='<plans>',
    type=int,
    default=None,
    help='New value for the "plans" quota. The default value is 50.')
def do_quota_class_update(cs, args):
    """Update the quotas for a quota class (Admin only)."""
    class_name = args.class_name
    data = {
        "plans": args.plans,
    }
    result = cs.quota_classes.update(class_name, data)
    _quota_set_pretty_show(result)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Data protection V1 os-services action implementations"""

from osc_lib.command import command
from osc_lib import utils as osc_utils
from oslo_log import log as logging

from karborclient.i18n import _


class ListServices(command.Lister):
    _description = _("List services.")

    log = logging.getLogger(__name__ + ".ListServices")

    def get_parser(self, prog_name):
        parser = super(ListServices, self).get_parser(prog_name)
        parser.add_argument(
            '--host',
            metavar='<host>',
            help=_('Filter results by host'),
        )
        parser.add_argument(
            '--binary',
            metavar='<binary>',
            help=_('Filter results by binary'),
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        data_protection_client = self.app.client_manager.data_protection
        data = data_protection_client.services.list(
            host=parsed_args.host,
            binary=parsed_args.binary
        )

        column_headers = ["Id", "Binary", "Host", "Status", "State",
                          "Updated_at", "Disabled Reason"]
        return (column_headers,
                (osc_utils.get_item_properties(
                    s, column_headers
                ) for s in data))


class EnableService(command.ShowOne):
    _description = _('Enable service')

    def get_parser(self, prog_name):
        parser = super(EnableService, self).get_parser(prog_name)
        parser.add_argument(
            'service_id',
            metavar='<service_id>',
            help=_('The ID of the service.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        service = client.services.enable(parsed_args.service_id)
        return zip(*sorted(service._info.items()))


class DisableService(command.ShowOne):
    _description = _('Disable service')

    def get_parser(self, prog_name):
        parser = super(DisableService, self).get_parser(prog_name)
        parser.add_argument(
            'service_id',
            metavar='<service_id>',
            help=_('The ID of the service.'),
        )
        parser.add_argument(
            '--reason',
            metavar='<reason>',
            help=_('Reason for disabling the service.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        if parsed_args.reason:
            service = client.services.disable_log_reason(
                parsed_args.service_id, parsed_args.reason)
        else:
            service = client.services.disable(parsed_args.service_id)
        return zip(*sorted(service._info.items()))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from karborclient.common import base


class ScheduledOperation(base.Resource):
    def __repr__(self):
        return "<ScheduledOperation %s>" % self._info


class ScheduledOperationManager(base.ManagerWithFind):
    resource_class = ScheduledOperation

    def create(self, name, operation_type, trigger_id, operation_definition):
        body = {'scheduled_operation': {'name': name,
                                        'operation_type': operation_type,
                                        'trigger_id': trigger_id,
                                        'operation_definition':
                                            operation_definition,
                                        }}
        url = "/scheduled_operations"
        return self._create(url, body, 'scheduled_operation')

    def delete(self, scheduled_operation_id):
        path = '/scheduled_operations/{scheduled_operation_id}'.\
            format(scheduled_operation_id=scheduled_operation_id)
        return self._delete(path)

    def get(self, scheduled_operation_id, session_id=None):
        if session_id:
            headers = {'X-Configuration-Session': session_id}
        else:
            headers = {}
        url = "/scheduled_operations/{scheduled_operation_id}".\
            format(scheduled_operation_id=scheduled_operation_id)
        return self._get(url, response_key="scheduled_operation",
                         headers=headers)

    def list(self, detailed=False, search_opts=None, marker=None, limit=None,
             sort_key=None, sort_dir=None, sort=None):
        """Lists all scheduled_operations."""

        resource_type = "scheduled_operations"
        url = self._build_list_url(
            resource_type, detailed=detailed,
            search_opts=search_opts, marker=marker,
            limit=limit, sort_key=sort_key,
            sort_dir=sort_dir, sort=sort)
        return self._list(url, 'operations')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Data protection V1 quota classes action implementations"""

from osc_lib.command import command


def quota_class_set_pretty_show(quota_classes):
    """Convert quotas class object to dict and display."""

    new_quota_classes = []
    for quota_k, quota_v in sorted(quota_classes.to_dict().items()):
        if isinstance(quota_v, dict):
            quota_v = '\n'.join(
                ['%s = %s' % (k, v) for k, v in sorted(quota_v.items())])
        new_quota_classes.append((quota_k, quota_v))

    return new_quota_classes


class ShowQuotaClasses(command.ShowOne):
    _description = "Shows Quota classes."

    def get_parser(self, prog_name):
        parser = super(ShowQuotaClasses, self).get_parser(prog_name)
        parser.add_argument(
            'class_name',
            metavar='<class_name>',
            help='Name of quota class to list the quotas for.')
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        result = client.quota_classes.get(parsed_args.class_name)
        quota_classes = quota_class_set_pretty_show(result)
        return zip(*sorted(quota_classes))


class UpdateQuotaClasses(command.ShowOne):
    _description = "Update the quotas for a quota class (Admin only)."

    def get_parser(self, prog_name):
        parser = super(UpdateQuotaClasses, self).get_parser(prog_name)
        parser.add_argument(
            'class_name',
            metavar='<class_name>',
            help='Name of quota class to set the quotas for.')
        parser.add_argument(
            '--plans',
            metavar='<plans>',
            type=int,
            default=None,
            help='New value for the "plans" quota.')
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        class_name = parsed_args.class_name
        data = {
            "plans": parsed_args.plans,
        }
        result = client.quota_classes.update(class_name, data)
        quota_classes = quota_class_set_pretty_show(result)

        return zip(*sorted(quota_classes))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from karborclient.common import base


class Provider(base.Resource):
    def __repr__(self):
        return "<Provider %s>" % self._info


class ProviderManager(base.ManagerWithFind):
    resource_class = Provider

    def get(self, provider_id, session_id=None):
        if session_id:
            headers = {'X-Configuration-Session': session_id}
        else:
            headers = {}
        url = "/providers/{provider_id}".format(
            provider_id=provider_id)
        return self._get(url, response_key="provider", headers=headers)

    def list(self, detailed=False, search_opts=None, marker=None, limit=None,
             sort_key=None, sort_dir=None, sort=None):
        """Lists all providers.

        :param detailed: Whether to return detailed provider info.
        :param search_opts: Search options to filter out provider.
        :param marker: Begin returning volumes that appear later in the
        provider  list than that represented by this provider id.
        :param limit: Maximum number of providers to return.
        :param sort_key: Key to be sorted; deprecated in kilo
        :param sort_dir: Sort direction, should be 'desc' or 'asc'; deprecated
                         in kilo
        :param sort: Sort information
        :rtype: list of :class:`Provider`
        """

        resource_type = "providers"
        url = self._build_list_url(
            resource_type, detailed=detailed,
            search_opts=search_opts, marker=marker,
            limit=limit, sort_key=sort_key,
            sort_dir=sort_dir, sort=sort)
        return self._list(url, 'providers')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Data protection V1 scheduled_operations action implementations"""

import functools
import six

from oslo_serialization import jsonutils
from oslo_utils import uuidutils

from osc_lib.command import command
from osc_lib import utils as osc_utils
from oslo_log import log as logging

from karborclient.common.apiclient import exceptions
from karborclient.i18n import _


def format_scheduledoperation(scheduledoperation_info):
    for key in ('operation_definition', ):
        if key not in scheduledoperation_info:
            continue
        scheduledoperation_info[key] = jsonutils.dumps(
            scheduledoperation_info[key], indent=2, sort_keys=True)
    scheduledoperation_info.pop("links", None)


class ListScheduledOperations(command.Lister):
    _description = _("List scheduled_operations.")

    log = logging.getLogger(__name__ + ".ListScheduledOperations")

    def get_parser(self, prog_name):
        parser = super(ListScheduledOperations, self).get_parser(prog_name)
        parser.add_argument(
            '--all-projects',
            action='store_true',
            default=False,
            help=_('Shows details for all tenants. Admin only.'),
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Filters results by a name. Default=None.'),
        )
        parser.add_argument(
            '--operation_type',
            metavar='<operation_type>',
            default=None,
            help=_('Filters results by a type. Default=None.'),
        )
        parser.add_argument(
            '--trigger_id',
            metavar='<trigger_id>',
            default=None,
            help=_('Filters results by a trigger id. Default=None.'),
        )
        parser.add_argument(
            '--operation_definition',
            metavar='<operation_definition>',
            default=None,
            help=_('Filters results by a operation definition. Default=None.'),
        )
        parser.add_argument(
            '--marker',
            metavar='<scheduled_operations>',
            help=_('The last scheduled_operations ID of the previous page'),
        )
        parser.add_argument(
            '--limit',
            type=int,
            metavar='<num-scheduled_operations>',
            help=_('Maximum number of scheduled_operations to display'),
        )
        parser.add_argument(
            '--sort',
            metavar="<key>[:<direction>]",
            default=None,
            help=_("Sort output by selected keys and directions(asc or desc) "
                   "(default: name:asc), multiple keys and directions can be "
                   "specified separated by comma"),
        )
        parser.add_argument(
            '--project',
            metavar='<project>',
            help=_('Filter results by a project(admin only)')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        data_protection_client = self.app.client_manager.data_protection
        all_projects = bool(parsed_args.project) or parsed_args.all_projects

        search_opts = {
            'all_tenants': all_projects,
            'project_id': parsed_args.project,
            'name': parsed_args.name,
            'operation_type': parsed_args.operation_type,
            'trigger_id': parsed_args.trigger_id,
            'operation_definition': parsed_args.operation_definition,
        }

        data = data_protection_client.scheduled_operations.list(
            search_opts=search_opts, marker=parsed_args.marker,
            limit=parsed_args.limit, sort=parsed_args.sort)

        column_headers = ['Id', 'Name', 'Operation Type', 'Trigger Id',
                          'Operation Definition']

        json_dumps = functools.partial(jsonutils.dumps,
                                       indent=2,
                                       sort_keys=True)
        formatters = {
            "Operation Definition": json_dumps,
        }
        return (column_headers,
                list(osc_utils.get_item_properties(
                    s, column_headers, formatters=formatters,
                ) for s in data))


class ShowScheduledOperation(command.ShowOne):
    _description = "Shows scheduled_operation details"

    def get_parser(self, prog_name):
        parser = super(ShowScheduledOperation, self).get_parser(prog_name)
        parser.add_argument(
            'scheduledoperation',
            metavar="<scheduledoperation>",
            help=_('The UUID of the scheduledoperation.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        so = osc_utils.find_resource(client.scheduled_operations,
                                     parsed_args.scheduledoperation)

        format_scheduledoperation(so._info)
        return zip(*sorted(six.iteritems(so._info)))


class CreateScheduledOperation(command.ShowOne):
    _description = "Creates a scheduled operation"

    def get_parser(self, prog_name):
        parser = super(CreateScheduledOperation, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('The name of the scheduled operation.')
        )
        parser.add_argument(
            'operation_type',
            metavar='<operation_type>',
            help=_('Operation Type of scheduled operation.')
        )
        parser.add_argument(
            'trigger_id',
            metavar='<trigger_id>',
            help=_('Trigger id of scheduled operation.')
        )
        parser.add_argument(
            'operation_definition',
            metavar='<key=value,key=value>',
            help=_('Operation definition of scheduled operation.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        if not uuidutils.is_uuid_like(parsed_args.trigger_id):
            raise exceptions.CommandError(
                "Invalid trigger id provided.")
        so = client.scheduled_operations.create(
            parsed_args.name, parsed_args.operation_type,
            parsed_args.trigger_id, parsed_args.operation_definition)

        format_scheduledoperation(so._info)
        return zip(*sorted(six.iteritems(so._info)))


class DeleteScheduledOperation(command.Command):
    _description = "Delete scheduled operation"

    def get_parser(self, prog_name):
        parser = super(DeleteScheduledOperation, self).get_parser(prog_name)
        parser.add_argument(
            'scheduledoperation',
            metavar='<scheduledoperation>',
            nargs="+",
            help=_('ID of scheduled operation.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        failure_count = 0
        for so_id in parsed_args.scheduledoperation:
            try:
                so = osc_utils.find_resource(client.scheduled_operations,
                                             so_id)
                client.scheduled_operations.delete(so.id)
            except exceptions.NotFound:
                failure_count += 1
                print("Failed to delete '%s'; scheduled operation "
                      "not found" % so_id)
        if failure_count == len(parsed_args.scheduledoperation):
            raise exceptions.CommandError(
                "Unable to find and delete any of the "
                "specified scheduled operation.")



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
import json

from karborclient.osc.v1 import scheduled_operations as osc_so
from karborclient.tests.unit.osc.v1 import fakes
from karborclient.v1 import scheduled_operations


SCHEDULEDOPERATION_INFO = {
    "id": "1a2c0c3d-f402-4cd8-b5db-82e85cb51fad",
    "name": "My scheduled operation",
    "description": "It will run everyday",
    "operation_type": "protect",
    "trigger_id": "23902b02-5666-4ee6-8dfe-962ac09c3995",
    "operation_definition": {
        "provider_id": "2a9ce1f3-cc1a-4516-9435-0ebb13caa399",
        "plan_id": "2a9ce1f3-cc1a-4516-9435-0ebb13caa398"
    },
    "enabled": 1
}


class TestScheduledOperations(fakes.TestDataProtection):
    def setUp(self):
        super(TestScheduledOperations, self).setUp()
        self.so_mock = self.app.client_manager.data_protection.\
            scheduled_operations
        self.so_mock.reset_mock()


class TestListScheduledOperations(TestScheduledOperations):
    def setUp(self):
        super(TestListScheduledOperations, self).setUp()
        self.so_mock.list.return_value = [
            scheduled_operations.ScheduledOperation(
                None, copy.deepcopy(SCHEDULEDOPERATION_INFO))
        ]

        # Command to test
        self.cmd = osc_so.ListScheduledOperations(self.app, None)

    def test_scheduled_operations_list(self):
        arglist = ['--name', 'My scheduled operation']
        verifylist = [('name', 'My scheduled operation')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = (
            ['Id', 'Name', 'Operation Type', 'Trigger Id',
             'Operation Definition'])
        self.assertEqual(expected_columns, columns)

        operation_definition = {
            "provider_id": "2a9ce1f3-cc1a-4516-9435-0ebb13caa399",
            "plan_id": "2a9ce1f3-cc1a-4516-9435-0ebb13caa398"
        }

        # Check that data is correct
        expected_data = [("1a2c0c3d-f402-4cd8-b5db-82e85cb51fad",
                          "My scheduled operation",
                          "protect",
                          "23902b02-5666-4ee6-8dfe-962ac09c3995",
                          json.dumps(operation_definition,
                                     indent=2, sort_keys=True)
                          )]
        self.assertEqual(expected_data, data)


class TestCreateScheduledOperation(TestScheduledOperations):
    def setUp(self):
        super(TestCreateScheduledOperation, self).setUp()
        self.so_mock.create.return_value = scheduled_operations.\
            ScheduledOperation(None, copy.deepcopy(SCHEDULEDOPERATION_INFO))
        # Command to test
        self.cmd = osc_so.CreateScheduledOperation(self.app, None)

    def test_scheduled_operation_create(self):
        arglist = ['My scheduled operation',
                   'protect',
                   "23902b02-5666-4ee6-8dfe-962ac09c3995",
                   "'provider_id=2a9ce1f3-cc1a-4516-9435-0ebb13caa399,"
                   "plan_id=2a9ce1f3-cc1a-4516-9435-0ebb13caa398'"]
        verifylist = [('name', 'My scheduled operation'),
                      ('operation_type', 'protect'),
                      ('trigger_id', "23902b02-5666-4ee6-8dfe-962ac09c3995"),
                      ('operation_definition',
                       "'provider_id=2a9ce1f3-cc1a-4516-9435-0ebb13caa399,"
                       "plan_id=2a9ce1f3-cc1a-4516-9435-0ebb13caa398'")]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.so_mock.create.assert_called_once_with(
            'My scheduled operation',
            'protect',
            '23902b02-5666-4ee6-8dfe-962ac09c3995',
            "'provider_id=2a9ce1f3-cc1a-4516-9435-0ebb13caa399,"
            "plan_id=2a9ce1f3-cc1a-4516-9435-0ebb13caa398'")


class TestDeleteScheduledOperation(TestScheduledOperations):
    def setUp(self):
        super(TestDeleteScheduledOperation, self).setUp()
        self.so_mock.get.return_value = scheduled_operations.\
            ScheduledOperation(None, copy.deepcopy(SCHEDULEDOPERATION_INFO))
        # Command to test
        self.cmd = osc_so.DeleteScheduledOperation(self.app, None)

    def test_scheduled_operation_delete(self):
        arglist = ['1a2c0c3d-f402-4cd8-b5db-82e85cb51fad']
        verifylist = [('scheduledoperation',
                       ['1a2c0c3d-f402-4cd8-b5db-82e85cb51fad'])]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        # Check that correct arguments were passed
        self.so_mock.delete.assert_called_once_with(
            '1a2c0c3d-f402-4cd8-b5db-82e85cb51fad')


class TestShowScheduledOperation(TestScheduledOperations):
    def setUp(self):
        super(TestShowScheduledOperation, self).setUp()
        self._schedop_info = copy.deepcopy(SCHEDULEDOPERATION_INFO)
        self.so_mock.get.return_value = scheduled_operations.\
            ScheduledOperation(None, self._schedop_info)

        # Command to test
        self.cmd = osc_so.ShowScheduledOperation(self.app, None)

    def test_scheduled_operation_show(self):
        arglist = ['1a2c0c3d-f402-4cd8-b5db-82e85cb51fad']
        verifylist = [('scheduledoperation',
                       '1a2c0c3d-f402-4cd8-b5db-82e85cb51fad')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Check that columns are correct
        expected_columns = (
            'description', 'enabled', 'id', 'name', 'operation_definition',
            'operation_type', 'trigger_id')
        self.assertEqual(expected_columns, columns)

        # Check that data is correct
        self.assertEqual(self._schedop_info['description'], data[0])
        self.assertEqual(self._schedop_info['enabled'], data[1])
        self.assertEqual(self._schedop_info['id'], data[2])
        self.assertEqual(self._schedop_info['name'], data[3])
        self.assertEqual(self._schedop_info['operation_definition'], data[4])
        self.assertEqual(self._schedop_info['operation_type'], data[5])
        self.assertEqual(self._schedop_info['trigger_id'], data[6])



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from karborclient.osc.v1 import quota_classes as osc_quota_classes
from karborclient.tests.unit.osc.v1 import fakes
from karborclient.v1 import quota_classes


QUOTA_CLASSES_INFO = {
    "id": "default",
    "plans": "40"
}


class TestQuotaClasses(fakes.TestDataProtection):
    def setUp(self):
        super(TestQuotaClasses, self).setUp()
        self.quotas_mock = (
            self.app.client_manager.data_protection.quota_classes)
        self.quotas_mock.reset_mock()


class TestUpdateQuotaClasses(TestQuotaClasses):
    def setUp(self):
        super(TestUpdateQuotaClasses, self).setUp()
        self.quotas_mock.update.return_value = quota_classes.QuotaClass(
            None, copy.deepcopy(QUOTA_CLASSES_INFO))
        self.cmd = osc_quota_classes.UpdateQuotaClasses(self.app, None)

    def test_quota_classes_update(self):
        arglist = ['--plans',
                   '40', 'default']
        verifylist = [('plans', 40),
                      ('class_name',
                       'default')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)

        self.quotas_mock.update.assert_called_once_with(
            'default',
            {'plans': 40})


class TestShowQuotaClasses(TestQuotaClasses):
    def setUp(self):
        super(TestShowQuotaClasses, self).setUp()
        self._quota_info = copy.deepcopy(QUOTA_CLASSES_INFO)
        self.quotas_mock.get.return_value = quota_classes.QuotaClass(
            None, copy.deepcopy(QUOTA_CLASSES_INFO))

        self.cmd = osc_quota_classes.ShowQuotaClasses(self.app, None)

    def test_quota_classes_show(self):
        arglist = ['default']
        verifylist = [('class_name', 'default')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(('id', 'plans'), columns)
        self.assertEqual(('default', '40'),
                         data)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import mock

from karborclient.tests.unit import base
from karborclient.tests.unit.v1 import fakes

cs = fakes.FakeClient()
mock_request_return = ({}, {'service': {}})


class ServicesTest(base.TestCaseShell):

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_services(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.services.list()
        mock_request.assert_called_with(
            'GET',
            '/os-services',
            headers={}
        )

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_services_with_host(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.services.list(host='fake_host')
        mock_request.assert_called_with(
            'GET',
            '/os-services?host=fake_host',
            headers={}
        )

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_services_with_binary(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.services.list(binary='fake_binary')
        mock_request.assert_called_with(
            'GET',
            '/os-services?binary=fake_binary',
            headers={}
        )

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_services_with_host_and_binary(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.services.list(host='fake_host', binary='fake_binary')
        mock_request.assert_called_with(
            'GET',
            '/os-services?binary=fake_binary&host=fake_host',
            headers={}
        )

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_enable_service(self, mock_request):
        mock_request.return_value = mock_request_return
        body = {
            'status': 'enabled'
        }
        cs.services.enable('1')
        mock_request.assert_called_with(
            'PUT',
            '/os-services/1',
            data=body,
            headers={}
        )

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_disable_service(self, mock_request):
        mock_request.return_value = mock_request_return
        body = {
            'status': 'disabled'
        }
        cs.services.disable('1')
        mock_request.assert_called_with(
            'PUT',
            '/os-services/1',
            data=body,
            headers={}
        )

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_disable_service_with_reason(self, mock_request):
        mock_request.return_value = mock_request_return
        body = {
            'status': 'disabled',
            'disabled_reason': 'fake_reason'
        }
        cs.services.disable_log_reason('1', 'fake_reason')
        mock_request.assert_called_with(
            'PUT',
            '/os-services/1',
            data=body,
            headers={}
        )



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from karborclient.common import base


class Verification(base.Resource):
    def __repr__(self):
        return "<Verification %s>" % self._info


class VerificationManager(base.ManagerWithFind):
    resource_class = Verification

    def create(self, provider_id, checkpoint_id, parameters):
        body = {
            'verification': {
                'provider_id': provider_id,
                'checkpoint_id': checkpoint_id,
                'parameters': parameters,
            }
        }
        url = "/verifications"
        return self._create(url, body, 'verification')

    def get(self, verification_id, session_id=None):
        if session_id:
            headers = {'X-Configuration-Session': session_id}
        else:
            headers = {}
        url = "/verifications/{verification_id}".format(
            verification_id=verification_id)
        return self._get(url, response_key="verification", headers=headers)

    def list(self, detailed=False, search_opts=None, marker=None, limit=None,
             sort_key=None, sort_dir=None, sort=None):
        """Lists all verifications.

        :param detailed: Whether to return detailed verification info.
        :param search_opts: Search options to filter out verifications.
        :param marker: Begin returning verifications that appear later in the
                       list than that represented by this verification id.
        :param limit: Maximum number of verifications to return.
        :param sort_key: Key to be sorted;
        :param sort_dir: Sort direction, should be 'desc' or 'asc';
        :param sort: Sort information
        :rtype: list of :class:`Verification`
        """

        resource_type = "verifications"
        url = self._build_list_url(
            resource_type, detailed=detailed,
            search_opts=search_opts, marker=marker,
            limit=limit, sort_key=sort_key,
            sort_dir=sort_dir, sort=sort)
        return self._list(url, 'verifications')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Command-line interface to the karbor Project.
"""

from __future__ import print_function

import argparse
import copy
import sys

from keystoneauth1 import discover
from keystoneauth1 import exceptions as ks_exc
from keystoneauth1.identity.generic import password
from keystoneauth1.identity.generic import token
from keystoneauth1 import loading
from oslo_log import handlers
from oslo_log import log as logging
from oslo_utils import encodeutils
from oslo_utils import importutils

import six
import six.moves.urllib.parse as urlparse

import karborclient
from karborclient import client as karbor_client
from karborclient.common.apiclient import exceptions as exc
from karborclient.common import utils


logger = logging.getLogger(__name__)


class KarborShell(object):

    def _append_global_identity_args(self, parser, argv):
        loading.register_session_argparse_arguments(parser)
        # Peek into argv to see if os-auth-token (or the deprecated
        # os_auth_token) or the new os-token or the environment variable
        # OS_AUTH_TOKEN were given. In which case, the token auth plugin is
        # what the user wants. Else, we'll default to password.
        default_auth_plugin = 'password'
        token_opts = ['os-token', 'os-auth-token', 'os_auth-token']
        if argv and any(i in token_opts for i in argv):
            default_auth_plugin = 'token'
        loading.register_auth_argparse_arguments(
            parser, argv, default=default_auth_plugin)

    def get_base_parser(self, argv):

        parser = argparse.ArgumentParser(
            prog='karbor',
            description=__doc__.strip(),
            epilog='See "karbor help COMMAND" '
                   'for help on a specific command.',
            add_help=False,
            formatter_class=HelpFormatter,
        )

        # Global arguments
        parser.add_argument('-h', '--help',
                            action='store_true',
                            help=argparse.SUPPRESS, )

        parser.add_argument('--version',
                            action='version',
                            version=karborclient.__version__,
                            help="Show program's version number and exit.")

        parser.add_argument('-d', '--debug',
                            default=bool(utils.env('KARBORCLIENT_DEBUG')),
                            action='store_true',
                            help='Defaults to env[KARBORCLIENT_DEBUG].')

        parser.add_argument('-v', '--verbose',
                            default=False, action="store_true",
                            help="Print more verbose output.")

        # os-cert, os-key, insecure, ca-file are all added
        # by keystone session register_cli_opts later
        parser.add_argument('--cert-file',
                            dest='os_cert',
                            help='DEPRECATED! Use --os-cert.')

        parser.add_argument('--key-file',
                            dest='os_key',
                            help='DEPRECATED! Use --os-key.')

        parser.add_argument('--ca-file',
                            dest='os_cacert',
                            help='DEPRECATED! Use --os-cacert.')

        parser.add_argument('--api-timeout',
                            help='Number of seconds to wait for an '
                                 'API response, '
                                 'defaults to system socket timeout.')

        parser.add_argument('--os_tenant_id',
                            default=utils.env('OS_TENANT_ID'),
                            help='Defaults to env[OS_TENANT_ID].')

        parser.add_argument('--os_tenant_name',
                            default=utils.env('OS_TENANT_NAME'),
                            help='Defaults to env[OS_TENANT_NAME].')

        parser.add_argument('--os-region-name',
                            default=utils.env('OS_REGION_NAME'),
                            help='Defaults to env[OS_REGION_NAME].')

        parser.add_argument('--os-auth-token',
                            default=utils.env('OS_AUTH_TOKEN'),
                            help='Defaults to env[OS_AUTH_TOKEN].')

        parser.add_argument('--os-no-client-auth',
                            default=utils.env('OS_NO_CLIENT_AUTH'),
                            action='store_true',
                            help="Do not contact keystone for a token. "
                                 "Defaults to env[OS_NO_CLIENT_AUTH].")
        parser.add_argument('--karbor-url',
                            default=utils.env('KARBOR_URL'),
                            help='Defaults to env[KARBOR_URL].')

        parser.add_argument('--karbor-api-version',
                            default=utils.env(
                                'KARBOR_API_VERSION', default='1'),
                            help='Defaults to env[KARBOR_API_VERSION] '
                                 'or 1.')

        parser.add_argument('--os-service-type',
                            default=utils.env('OS_SERVICE_TYPE'),
                            help='Defaults to env[OS_SERVICE_TYPE].')

        parser.add_argument('--os-endpoint-type',
                            default=utils.env('OS_ENDPOINT_TYPE'),
                            help='Defaults to env[OS_ENDPOINT_TYPE].')

        parser.add_argument('--include-password',
                            default=bool(utils.env('KARBOR_INCLUDE_PASSWORD')),
                            action='store_true',
                            help='Send os-username and os-password to karbor.')

        self._append_global_identity_args(parser, argv)

        return parser

    def get_subcommand_parser(self, version, argv=None):
        parser = self.get_base_parser(argv)

        self.subcommands = {}
        subparsers = parser.add_subparsers(metavar='<subcommand>')
        submodule = importutils.import_versioned_module(
            'karborclient', version, 'shell'
        )
        self._find_actions(subparsers, submodule)
        self._find_actions(subparsers, self)

        return parser

    def _find_actions(self, subparsers, actions_module):
        for attr in (a for a in dir(actions_module) if a.startswith('do_')):
            # I prefer to be hypen-separated instead of underscores.
            command = attr[3:].replace('_', '-')
            callback = getattr(actions_module, attr)
            desc = callback.__doc__ or ''
            help = desc.strip().split('\n')[0]
            arguments = getattr(callback, 'arguments', [])

            subparser = subparsers.add_parser(command, help=help,
                                              description=desc,
                                              add_help=False,
                                              formatter_class=HelpFormatter)
            subparser.add_argument('-h', '--help', action='help',
                                   help=argparse.SUPPRESS)
            self.subcommands[command] = subparser
            for (args, kwargs) in arguments:
                subparser.add_argument(*args, **kwargs)
            subparser.set_defaults(func=callback)

    def _discover_auth_versions(self, session, auth_url):
        # discover the API versions the server is supporting base on the
        # given URL
        v2_auth_url = None
        v3_auth_url = None
        try:
            ks_discover = discover.Discover(session=session, url=auth_url)
            v2_auth_url = ks_discover.url_for('2.0')
            v3_auth_url = ks_discover.url_for('3.0')
        except ks_exc.ClientException as e:
            # Identity service may not support discover API version.
            # Lets trying to figure out the API version from the original URL.
            url_parts = urlparse.urlparse(auth_url)
            (scheme, netloc, path, params, query, fragment) = url_parts
            path = path.lower()
            if path.startswith('/v3'):
                v3_auth_url = auth_url
            elif path.startswith('/v2'):
                v2_auth_url = auth_url
            else:
                # not enough information to determine the auth version
                msg = ('Unable to determine the Keystone version '
                       'to authenticate with using the given '
                       'auth_url. Identity service may not support API '
                       'version discovery. Please provide a versioned '
                       'auth_url instead. error=%s') % (e)
                raise exc.CommandError(msg)

        return (v2_auth_url, v3_auth_url)

    def _get_keystone_auth(self, session, auth_url, **kwargs):
        auth_token = kwargs.pop('auth_token', None)
        if auth_token:
            return token.Token(
                auth_url,
                auth_token,
                project_id=kwargs.pop('project_id'),
                project_name=kwargs.pop('project_name'),
                project_domain_id=kwargs.pop('project_domain_id'),
                project_domain_name=kwargs.pop('project_domain_name'))

        # NOTE(starodubcevna): this is a workaround for the bug:
        # https://bugs.launchpad.net/python-openstackclient/+bug/1447704
        # Change that fix this error in keystoneclient was abandoned,
        # so we should use workaround until we move to keystoneauth.
        # The idea of the code came from glanceclient.

        (v2_auth_url, v3_auth_url) = self._discover_auth_versions(
            session=session,
            auth_url=auth_url)

        if v3_auth_url:
            # NOTE(starodubcevna): set user_domain_id and project_domain_id
            # to default as it done in other projects.
            return password.Password(auth_url,
                                     username=kwargs.pop('username'),
                                     user_id=kwargs.pop('user_id'),
                                     password=kwargs.pop('password'),
                                     user_domain_id=kwargs.pop(
                                         'user_domain_id') or 'default',
                                     user_domain_name=kwargs.pop(
                                         'user_domain_name'),
                                     project_id=kwargs.pop('project_id'),
                                     project_name=kwargs.pop('project_name'),
                                     project_domain_id=kwargs.pop(
                                         'project_domain_id') or 'default')
        elif v2_auth_url:
            return password.Password(auth_url,
                                     username=kwargs.pop('username'),
                                     user_id=kwargs.pop('user_id'),
                                     password=kwargs.pop('password'),
                                     project_id=kwargs.pop('project_id'),
                                     project_name=kwargs.pop('project_name'))
        else:
            # if we get here it means domain information is provided
            # (caller meant to use Keystone V3) but the auth url is
            # actually Keystone V2. Obviously we can't authenticate a V3
            # user using V2.
            exc.CommandError("Credential and auth_url mismatch. The given "
                             "auth_url is using Keystone V2 endpoint, which "
                             "may not able to handle Keystone V3 credentials. "
                             "Please provide a correct Keystone V3 auth_url.")

    def _setup_logging(self, debug):
        # Output the logs to command-line interface
        color_handler = handlers.ColorHandler(sys.stdout)
        logger_root = logging.getLogger(None).logger
        logger_root.level = logging.DEBUG if debug else logging.WARNING
        logger_root.addHandler(color_handler)

        # Set the logger level of special library
        logging.getLogger('iso8601') \
            .logger.setLevel(logging.WARNING)
        logging.getLogger('urllib3.connectionpool') \
            .logger.setLevel(logging.WARNING)

    def main(self, argv):
        # Parse args once to find version
        base_argv = copy.deepcopy(argv)
        parser = self.get_base_parser(argv)
        (options, args) = parser.parse_known_args(base_argv)
        self._setup_logging(options.debug)

        # build available subcommands based on version
        api_version = options.karbor_api_version
        subcommand_parser = self.get_subcommand_parser(api_version, argv)
        self.parser = subcommand_parser

        ks_session = None
        keystone_auth = None

        # Handle top-level --help/-h before attempting to parse
        # a command off the command line.
        if (not args and options.help) or not argv:
            self.do_help(options)
            return 0

        # Parse args again and call whatever callback was selected.
        args = subcommand_parser.parse_args(argv)

        # Short-circuit and deal with help command right away.
        if args.func == self.do_help:
            self.do_help(args)
            return 0
        elif args.func == self.do_bash_completion:
            self.do_bash_completion(args)
            return 0

        if not args.os_username and not args.os_auth_token:
            raise exc.CommandError("You must provide a username via"
                                   " either --os-username or env[OS_USERNAME]"
                                   " or a token via --os-auth-token or"
                                   " env[OS_AUTH_TOKEN]")

        if args.os_no_client_auth:
            if not args.karbor_url:
                raise exc.CommandError(
                    "If you specify --os-no-client-auth"
                    " you must also specify a Karbor API URL"
                    " via either --karbor-url or env[KARBOR_URL]")

        else:
            # Tenant name or ID is needed to make keystoneclient retrieve a
            # service catalog, it's not required if os_no_client_auth is
            # specified, neither is the auth URL.
            if not any([args.os_tenant_name, args.os_tenant_id,
                        args.os_project_id, args.os_project_name]):
                raise exc.CommandError("You must provide a project name or"
                                       " project id via --os-project-name,"
                                       " --os-project-id, env[OS_PROJECT_ID]"
                                       " or env[OS_PROJECT_NAME]. You may"
                                       " use os-project and os-tenant"
                                       " interchangeably.")
            if not args.os_auth_url:
                raise exc.CommandError("You must provide an auth url via"
                                       " either --os-auth-url or via"
                                       " env[OS_AUTH_URL]")

        endpoint = args.karbor_url

        if args.os_no_client_auth:
            # Authenticate through karbor, don't use session
            kwargs = {
                'username': args.os_username,
                'password': args.os_password,
                'auth_token': args.os_auth_token,
                'auth_url': args.os_auth_url,
                'token': args.os_auth_token,
                'insecure': args.insecure,
                'timeout': args.api_timeout
            }

            if args.os_region_name:
                kwargs['region_name'] = args.os_region_name
        else:
            # Create a keystone session and keystone auth
            ks_session = loading.load_session_from_argparse_arguments(args)
            project_id = args.os_project_id or args.os_tenant_id
            project_name = args.os_project_name or args.os_tenant_name

            keystone_auth = self._get_keystone_auth(
                ks_session,
                args.os_auth_url,
                username=args.os_username,
                user_id=args.os_user_id,
                user_domain_id=args.os_user_domain_id,
                user_domain_name=args.os_user_domain_name,
                password=args.os_password,
                auth_token=args.os_auth_token,
                project_id=project_id,
                project_name=project_name,
                project_domain_id=args.os_project_domain_id,
                project_domain_name=args.os_project_domain_name)

            endpoint_type = args.os_endpoint_type or 'publicURL'
            service_type = args.os_service_type or 'data-protect'

            endpoint = keystone_auth.get_endpoint(
                ks_session,
                service_type=service_type,
                region_name=args.os_region_name)

            kwargs = {
                'session': ks_session,
                'auth': keystone_auth,
                'service_type': service_type,
                'endpoint_type': endpoint_type,
                'region_name': args.os_region_name,
            }

        if args.api_timeout:
            kwargs['timeout'] = args.api_timeout

        self.cs = karbor_client.Client(api_version, endpoint, **kwargs)

        args.func(self.cs, args)

    def do_bash_completion(self, args):
        """Prints all of the commands and options to stdout."""
        commands = set()
        options = set()
        for sc_str, sc in self.subcommands.items():
            commands.add(sc_str)
            for option in list(sc._optionals._option_string_actions):
                options.add(option)

        commands.remove('bash-completion')
        print(' '.join(commands | options))

    @utils.arg('command', metavar='<subcommand>', nargs='?',
               help='Display help for <subcommand>')
    def do_help(self, args):
        """Display help about this program or one of its subcommands.

        """
        if getattr(args, 'command', None):
            if args.command in self.subcommands:
                self.subcommands[args.command].print_help()
            else:
                msg = "'%s' is not a valid subcommand"
                raise exc.CommandError(msg % args.command)
        else:
            self.parser.print_help()


class HelpFormatter(argparse.HelpFormatter):
    def start_section(self, heading):
        # Title-case the headings
        heading = '%s%s' % (heading[0].upper(), heading[1:])
        super(HelpFormatter, self).start_section(heading)


def main(args=sys.argv[1:]):
    try:
        KarborShell().main(args)

    except KeyboardInterrupt:
        print('... terminating karbor client', file=sys.stderr)
        sys.exit(1)
    except Exception as e:
        if '--debug' in args or '-d' in args:
            raise
        else:
            print(encodeutils.safe_encode(six.text_type(e)), file=sys.stderr)
        sys.exit(1)


if __name__ == "__main__":
    main()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Data protection V1 protectables action implementations"""

import functools
from osc_lib.command import command
from osc_lib import utils as osc_utils
from oslo_log import log as logging
from oslo_serialization import jsonutils

from karborclient.i18n import _
from karborclient import utils


class ListProtectables(command.Lister):
    _description = _("List protectable types.")

    log = logging.getLogger(__name__ + ".ListProtectables")

    def get_parser(self, prog_name):
        parser = super(ListProtectables, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        data_protection_client = self.app.client_manager.data_protection

        data = data_protection_client.protectables.list()

        column_headers = ['Protectable type']

        return (column_headers,
                (osc_utils.get_item_properties(
                    s, column_headers
                ) for s in data))


class ShowProtectable(command.ShowOne):
    _description = "Shows protectable type details"

    def get_parser(self, prog_name):
        parser = super(ShowProtectable, self).get_parser(prog_name)
        parser.add_argument(
            'protectable_type',
            metavar="<protectable_type>",
            help=_('Protectable type.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        protectable = osc_utils.find_resource(client.protectables,
                                              parsed_args.protectable_type)

        protectable._info.pop("links", None)
        if 'dependent_types' in protectable._info:
            protectable._info['dependent_types'] = "\n".join(
                protectable._info['dependent_types'])
        return zip(*sorted(protectable._info.items()))


class ListProtectableInstances(command.Lister):
    _description = _("List protectable instances.")

    log = logging.getLogger(__name__ + ".ListProtectableInstances")

    def get_parser(self, prog_name):
        parser = super(ListProtectableInstances, self).get_parser(prog_name)
        parser.add_argument(
            'protectable_type',
            metavar="<protectable_type>",
            help=_('Type of protectable.')
        )
        parser.add_argument(
            '--type',
            metavar="<type>",
            default=None,
            help=_('Filters results by protectable type. Default=None.')
        )
        parser.add_argument(
            '--marker',
            metavar="<protectable_instance>",
            default=None,
            help=_('The last protectable instance ID of the previous page.')
        )
        parser.add_argument(
            '--limit',
            metavar="<num-protectable_instances>",
            default=None,
            help=_('Maximum number of protectable instances to display.')
        )
        parser.add_argument(
            '--sort',
            metavar="<key>[:<direction>]",
            default=None,
            help=_("Sort output by selected keys and directions(asc or desc), "
                   "multiple keys and directions can be "
                   "specified separated by comma"),
        )
        parser.add_argument(
            '--parameters',
            type=str,
            nargs='*',
            metavar="<key=value>",
            default=None,
            help=_('List instances by parameters key and value pair. '
                   'Default=None.')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        data_protection_client = self.app.client_manager.data_protection

        search_opts = {
            'type': parsed_args.type,
            'parameters': (utils.extract_instances_parameters(parsed_args)
                           if parsed_args.parameters else None),
        }

        data = data_protection_client.protectables.list_instances(
            parsed_args.protectable_type, search_opts=search_opts,
            marker=parsed_args.marker, limit=parsed_args.limit,
            sort=parsed_args.sort)

        column_headers = ['Id', 'Type', 'Name', 'Dependent resources',
                          'Extra info']

        json_dumps = functools.partial(jsonutils.dumps,
                                       indent=2, sort_keys=True)
        formatters = {
            "Extra info": json_dumps,
            "Dependent resources": json_dumps,
        }
        return (column_headers,
                (osc_utils.get_item_properties(
                    s, column_headers, formatters=formatters,
                ) for s in data))


class ShowProtectableInstance(command.ShowOne):
    _description = "Shows protectable instance details"

    def get_parser(self, prog_name):
        parser = super(ShowProtectableInstance, self).get_parser(prog_name)
        parser.add_argument(
            'protectable_type',
            metavar="<protectable_type>",
            help=_('Protectable type.')
        )
        parser.add_argument(
            'protectable_id',
            metavar="<protectable_id>",
            help=_('Protectable instance id.')
        )
        parser.add_argument(
            '--parameters',
            type=str,
            nargs='*',
            metavar="<key=value>",
            default=None,
            help=_('Show a instance by parameters key and value pair. '
                   'Default=None.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection

        search_opts = {
            'parameters': (utils.extract_instances_parameters(parsed_args)
                           if parsed_args.parameters else None),
        }

        instance = client.protectables.get_instance(
            parsed_args.protectable_type,
            parsed_args.protectable_id,
            search_opts=search_opts)

        json_dumps = functools.partial(jsonutils.dumps,
                                       indent=2, sort_keys=True)
        instance._info.pop("links", None)
        for key in ('extra_info', 'dependent_resources'):
            if key not in instance._info:
                continue
            instance._info[key] = json_dumps(instance._info[key])

        return zip(*sorted(instance._info.items()))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Data protection V1 triggers action implementations"""

from osc_lib.command import command
from osc_lib import utils as osc_utils
from oslo_log import log as logging

from karborclient.common.apiclient import exceptions
from karborclient.i18n import _
from karborclient import utils


class ListTriggers(command.Lister):
    _description = _("List triggers.")

    log = logging.getLogger(__name__ + ".ListTriggers")

    def get_parser(self, prog_name):
        parser = super(ListTriggers, self).get_parser(prog_name)
        parser.add_argument(
            '--all-projects',
            action='store_true',
            default=False,
            help=_('Shows details for all tenants. Admin only.'),
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            default=None,
            help=_('Filters results by a name. Default=None.'),
        )
        parser.add_argument(
            '--type',
            metavar='<type>',
            default=None,
            help=_('Filters results by a type. Default=None.'),
        )
        parser.add_argument(
            '--properties',
            metavar='<properties>',
            default=None,
            help=_('Filters results by a properties. Default=None.'),
        )
        parser.add_argument(
            '--marker',
            metavar='<trigger>',
            help=_('The last trigger ID of the previous page'),
        )
        parser.add_argument(
            '--limit',
            type=int,
            metavar='<num-triggers>',
            help=_('Maximum number of triggers to display'),
        )
        parser.add_argument(
            '--sort',
            metavar="<key>[:<direction>]",
            default=None,
            help=_("Sort output by selected keys and directions(asc or desc) "
                   "(default: name:asc), multiple keys and directions can be "
                   "specified separated by comma"),
        )
        parser.add_argument(
            '--project',
            metavar='<project>',
            help=_('Display information from single tenant (Admin only).')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        data_protection_client = self.app.client_manager.data_protection
        all_projects = bool(parsed_args.project) or parsed_args.all_projects

        search_opts = {
            'all_tenants': all_projects,
            'project_id': parsed_args.project,
            'name': parsed_args.name,
            'type': parsed_args.type,
            'properties': parsed_args.properties,
        }

        data = data_protection_client.triggers.list(
            search_opts=search_opts, marker=parsed_args.marker,
            limit=parsed_args.limit, sort=parsed_args.sort)

        column_headers = ['Id', 'Name', 'Type', 'Properties']

        return (column_headers,
                (osc_utils.get_item_properties(
                    s, column_headers
                ) for s in data))


class ShowTrigger(command.ShowOne):
    _description = "Shows trigger details"

    def get_parser(self, prog_name):
        parser = super(ShowTrigger, self).get_parser(prog_name)
        parser.add_argument(
            'trigger',
            metavar="<trigger>",
            help=_('The UUID of the trigger.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        trigger = osc_utils.find_resource(client.triggers, parsed_args.trigger)

        trigger._info.pop("links", None)
        return zip(*sorted(trigger._info.items()))


class CreateTrigger(command.ShowOne):
    _description = "Creates a trigger"

    def get_parser(self, prog_name):
        parser = super(CreateTrigger, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('The name of the trigger.')
        )
        parser.add_argument(
            'type',
            metavar='<type>',
            help=_('Type of trigger.')
        )
        parser.add_argument(
            'properties',
            metavar='<key=value,key=value>',
            help=_('Properties of trigger.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        trigger = client.triggers.create(parsed_args.name, parsed_args.type,
                                         parsed_args.properties)

        trigger._info.pop("links", None)
        return zip(*sorted(trigger._info.items()))


class UpdateTrigger(command.ShowOne):
    _description = "Update a trigger"

    def get_parser(self, prog_name):
        parser = super(UpdateTrigger, self).get_parser(prog_name)
        parser.add_argument(
            "trigger_id",
            metavar="<TRIGGER ID>",
            help=_("Id of trigger to update.")
        )
        parser.add_argument(
            "--name",
            metavar="<name>",
            help=_("A name to which the trigger will be renamed.")
        )
        parser.add_argument(
            "--properties",
            metavar="<key=value,key=value>",
            help=_("Properties of trigger which will be updated.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        data = {}
        if parsed_args.name is not None:
            data['name'] = parsed_args.name
        if parsed_args.properties is not None:
            trigger_properties = utils.extract_properties(parsed_args)
            data['properties'] = trigger_properties
        try:
            trigger = osc_utils.find_resource(client.triggers,
                                              parsed_args.trigger_id)
            trigger = client.triggers.update(trigger.id, data)
        except exceptions.NotFound:
            raise exceptions.CommandError(
                "Trigger %s not found" % parsed_args.trigger_id)
        else:
            trigger._info.pop("links", None)
            return zip(*sorted(trigger._info.items()))


class DeleteTrigger(command.Command):
    _description = "Delete trigger"

    log = logging.getLogger(__name__ + ".DeleteTrigger")

    def get_parser(self, prog_name):
        parser = super(DeleteTrigger, self).get_parser(prog_name)
        parser.add_argument(
            'trigger',
            metavar='<trigger>',
            nargs="+",
            help=_('ID of trigger.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.data_protection
        failure_count = 0
        for trigger_id in parsed_args.trigger:
            try:
                trigger = osc_utils.find_resource(client.triggers, trigger_id)
                client.triggers.delete(trigger.id)
            except exceptions.NotFound:
                failure_count += 1
                self.log.error(
                    "Failed to delete '{0}'; trigger not found".
                    format(trigger_id))
        if failure_count == len(parsed_args.trigger):
            raise exceptions.CommandError(
                "Unable to find and delete any of the "
                "specified trigger.")



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from karborclient.tests.unit import base
from karborclient.tests.unit.v1 import fakes

cs = fakes.FakeClient()
mock_request_return = ({}, {'verification': {}})


class VerificationsTest(base.TestCaseShell):

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_verifications_with_marker_limit(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.verifications.list(marker=1234, limit=2)
        mock_request.assert_called_with(
            'GET',
            '/verifications?limit=2&marker=1234', headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_verifications_with_sort_key_dir(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.verifications.list(sort_key='id', sort_dir='asc')
        mock_request.assert_called_with(
            'GET',
            '/verifications?'
            'sort_dir=asc&sort_key=id', headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_list_verifications_with_invalid_sort_key(self, mock_request):
        self.assertRaises(ValueError,
                          cs.verifications.list,
                          sort_key='invalid', sort_dir='asc')

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_create_verification(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.verifications.create('586cc6ce-e286-40bd-b2b5-dd32694d9944',
                                '2220f8b1-975d-4621-a872-fa9afb43cb6c',
                                '{}')
        mock_request.assert_called_with(
            'POST',
            '/verifications',
            data={
                'verification':
                {
                    'checkpoint_id': '2220f8b1-975d-4621-a872-fa9afb43cb6c',
                    'parameters': '{}',
                    'provider_id': '586cc6ce-e286-40bd-b2b5-dd32694d9944'
                }}, headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_show_verification(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.verifications.get('1')
        mock_request.assert_called_with(
            'GET',
            '/verifications/1',
            headers={})

    @mock.patch('karborclient.common.http.HTTPClient.json_request')
    def test_show_verification_with_headers(self, mock_request):
        mock_request.return_value = mock_request_return
        cs.verifications.get('1', session_id='fake_session_id')
        mock_request.assert_called_with(
            'GET',
            '/verifications/1',
            headers={'X-Configuration-Session': 'fake_session_id'})



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from karborclient.common import base


class QuotaClass(base.Resource):
    def __repr__(self):
        return "<QuotaClass %s>" % self._info


class QuotaClassManager(base.ManagerWithFind):
    resource_class = QuotaClass

    def list(self):
        pass

    def update(self, class_name, data):

        if "plans" in data and data["plans"] is None:
            data["plans"] = 50

        body = {"quota_class": data}

        return self._update('/quota_classes/{class_name}'
                            .format(class_name=class_name),
                            body, "quota_class")

    def get(self, class_name, session_id=None):
        if session_id:
            headers = {'X-Configuration-Session': session_id}
        else:
            headers = {}
        url = "/quota_classes/{class_name}".format(
            class_name=class_name)
        return self._get(url, response_key="quota_class", headers=headers)
